@@ -1,4 +1,4 @@
-/* $Id: dbz-v6.c,v 1.9 1998/09/21 10:33:32 src Exp $
+/* TAB P   $Id: dbz-v6.c,v 1.12 1999/04/08 10:48:51 src Exp $
 
   dbz.c  V6.1.1
 
@@ -61,6 +61,15 @@ to produce files with holes in them, which is a nuisance.)
 Tagged hash + offset fuzzy technique merged by Sang-yong Suh (Nov, 1997)
 
 $Log: dbz-v6.c,v $
+Revision 1.12  1999/04/08 10:48:51  src
+Extendeddbz to configure
+
+Revision 1.11  1999/04/08 06:51:47  src
+Runtime tagged hash - still a long way to go
+
+Revision 1.10  1999/03/07 15:41:32  src
+Removed DBZ_VERSION from configuration
+
 Revision 1.9  1998/09/21 10:33:32  src
 Getting rid of some warnings.
 
@@ -81,6 +90,7 @@ Changed fopen() to fopen_W() to support read/only mode *EK*
 
 #include "configdata.h" /*EK*/
 #include "clibrary.h" /*EK*/
+#include "newsconfig.h" /*EK*/
 
 #if STDC_HEADERS
 #  include <stdlib.h>
@@ -134,9 +144,16 @@ Changed fopen() to fopen_W() to support read/only mode *EK*
 
 static int dbzversion = 6;      /* for validating .dir file format */
 
+#ifdef DO_TAGGED_HASH  /*EK*/
+int do_tagged_hash = 1; /*EK*/
+#else
+int do_tagged_hash = 0; /*EK*/
+#endif
+
 #define of_t            long
 #define SOF             (sizeof(of_t))
 #define NOTFOUND        ((OFFSET_T)-1)
+
 #ifdef  DO_TAGGED_HASH
 
 #define OVERFLOW
@@ -144,6 +161,9 @@ static int dbzversion = 6;      /* for validating .dir file format */
 #include <limits.h>
 #endif
 
+#endif
+
+/* for DO_TAGGED_HASH */
 /* MAXDROPBITS is the maximum number of bits dropped from the offset value.
    The least significant bits are dropped.  The space is used to
    store hash additional bits, thereby increasing the possibility of the
@@ -195,7 +215,7 @@ static OFFSET_T tagbits;                /* pre-shifted tag mask */
 static OFFSET_T taghere;                /* pre-shifted tag-enable bit */
 static OFFSET_T tagboth;                /* tagbits|taghere */
 
-#endif  /* DO_TAGGED_HASH */
+/* end DO_TAGGED_HASH */
 
 #ifdef MAP_FILE
 #define MAP__ARG        (MAP_FILE | MAP_SHARED)
@@ -324,14 +344,16 @@ typedef struct {
 static BOOL opendb = FALSE;     /* Indicates if a database is currently open */
 static FILE *dirf;              /* descriptor for .dir file */
 static BOOL readonly;           /* database open read-only? */
-#ifdef  DO_TAGGED_HASH
+
+/* for DO_TAGGED_HASH */
 static FILE *basef;             /* descriptor for base file */
 static char *basefname;         /* name for not-yet-opened base file */
 static hash_table pagtab;       /* pag hash table, stores hash + offset */
-#else
+
+/* for !DO_TAGGED_HASH */
 static hash_table idxtab;       /* index hash table, used for data retrieval */
 static hash_table etab;         /* existance hash table, used for existance checks */
-#endif
+
 static BOOL dirty;              /* has a store() been done? */
 static erec empty_rec;          /* empty rec to compare against
 				   initalized in dbzinit */
@@ -343,24 +365,26 @@ static BOOL putcore(hash_table *tab);
 static BOOL getconf(FILE *df, dbzconfig *cp);
 static int  putconf(FILE *f, dbzconfig *cp);
 static void start(searcher *sp, const HASH hash, searcher *osp);
+
 #ifdef  DO_TAGGED_HASH
 static OFFSET_T search(searcher *sp);
 static BOOL set_pag(searcher *sp, OFFSET_T value);
 #else
 static BOOL search(searcher *sp);
 #endif
+
 static BOOL set(searcher *sp, hash_table *tab, void *value);
 
 /* file-naming stuff */
 static char dir[] = ".dir";
-#ifdef  DO_TAGGED_HASH
+
+/* for DO_TAGGED_HASH */
 static char pag[] = ".pag";
-#else
+/* for !DO_TAGGED_HASH */
 static char idx[] = ".index";
 static char exists[] = ".hash";
-#endif
 
-#ifdef  DO_TAGGED_HASH
+/* for DO_TAGGED_HASH */
 /*
  - dbzconfbase - reconfigure dbzconf from base file size.
  */
@@ -404,7 +428,7 @@ static void config_by_text_size(dbzconfig *c, of_t basesize) {
 	exit(1);
     }
 }
-#endif  /* DO_TAGGED_HASH */
+/* end DO_TAGGED_HASH */
 
 /*
  - create and truncate .pag, .idx, or .hash files
@@ -437,10 +461,10 @@ BOOL dbzfresh(const char *name, const long size, const int fillpercent)
     char *fn;
     dbzconfig c;
     FILE *f;
-#ifdef  DO_TAGGED_HASH
+    /* for DO_TAGGED_HASH*/
     struct stat sb;
     of_t m;
-#endif
+    /* end DO_TAGGED_HASH*/
 
     if (opendb) {
 	DEBUG(("dbzfresh: database already open\n"));
@@ -455,30 +479,30 @@ BOOL dbzfresh(const char *name, const long size, const int fillpercent)
     if (!getconf((FILE *)NULL, &c))
 	return FALSE;   /* "can't happen" */
 
-#ifdef  DO_TAGGED_HASH
-    /* and mess with it as specified */
-    if (size != 0)
-	c.tsize = size;
-    m = c.tagmask;
-    c.tagshift = 0;
-    while (!(m & 1)) {
-	m >>= 1;
-	c.tagshift++;
-    }
-    c.tagmask = m;
-    c.tagenb = (m << 1) & ~m;
-    c.dropbits = 0;
-    c.lenfuzzy = 0;
+    if (do_tagged_hash) {
+	/* and mess with it as specified */
+	if (size != 0)
+	    c.tsize = size;
+	m = c.tagmask;
+	c.tagshift = 0;
+	while (!(m & 1)) {
+	    m >>= 1;
+	    c.tagshift++;
+	}
+	c.tagmask = m;
+	c.tagenb = (m << 1) & ~m;
+	c.dropbits = 0;
+	c.lenfuzzy = 0;
 
-    /* if big enough basb file exists, update config */
-    if (stat(name, &sb) != -1)
-	config_by_text_size(&c, sb.st_size);
-#else
-    /* set the size as specified, make sure we get at least 2 bytes
-       of implicit hash */
-    if (size != 0)
-	c.tsize = size > (64 * 1024) ? size : 64 * 1024;
-#endif
+	/* if big enough basb file exists, update config */
+	if (stat(name, &sb) != -1)
+	    config_by_text_size(&c, sb.st_size);
+    } else {
+	/* set the size as specified, make sure we get at least 2 bytes
+	   of implicit hash */
+	if (size != 0)
+	    c.tsize = size > (64 * 1024) ? size : 64 * 1024;
+    }
 
     /* write it out */
     if ((fn = enstring(name, dir)) == NULL)
@@ -499,21 +523,21 @@ BOOL dbzfresh(const char *name, const long size, const int fillpercent)
     }
 
     /* create and truncate .pag, or .index/.hash files */
-#ifdef  DO_TAGGED_HASH
-    if (!create_truncate(name, pag))
-	return FALSE;
-#else
-    if (!create_truncate(name, idx))
-	return FALSE;
-    if (!create_truncate(name, exists))
-	return FALSE;
-#endif  /* DO_TAGGED_HASH */
+    if (do_tagged_hash) {
+	if (!create_truncate(name, pag))
+	    return FALSE;
+    } else {
+	if (!create_truncate(name, idx))
+	    return FALSE;
+	if (!create_truncate(name, exists))
+	    return FALSE;
+    }
 
     /* and punt to dbzinit for the hard work */
     return dbzinit(name);
 }
 
-#ifdef  DO_TAGGED_HASH
+/* for DO_TAGGED_HASH */
 /*
  - isprime - is a number prime?
  */
@@ -542,7 +566,7 @@ static BOOL isprime(long x) {
 
     return TRUE;
 }
-#endif
+/* end DO_TAGGED_HASH */
 
 /*
  * dbzsize  - what's a good table size to hold this many entries?
@@ -565,13 +589,13 @@ long dbzsize(const long contents) {
     if (n < (64 * 1024))
 	n = 64 * 1024;
 
-#ifdef  DO_TAGGED_HASH
-    if (!(n & 1))
-	n += 1;                 /* make it odd */
-    DEBUG(("dbzsize: tentative size %ld\n", n));
-    while (!isprime(n))         /* look for a prime */
-	n += 2;
-#endif
+    if (do_tagged_hash) {
+	if (!(n & 1))
+	    n += 1;             /* make it odd */
+	DEBUG(("dbzsize: tentative size %ld\n", n));
+	while (!isprime(n))     /* look for a prime */
+	    n += 2;
+    }
 
     DEBUG(("dbzsize: final size %ld\n", n));
     return n;
@@ -592,10 +616,10 @@ BOOL dbzagain(const char *name, const char *oldname)
     FILE *f;
     int newtable;
     OFFSET_T newsize;
-#ifdef  DO_TAGGED_HASH
+    /* for DO_TAGGED_HASH */
     long vtop;
     struct stat sb;
-#endif
+    /* end DO_TAGGED_HASH */
 
     if (opendb) {
 	DEBUG(("dbzagain: database already open\n"));
@@ -635,27 +659,27 @@ BOOL dbzagain(const char *name, const char *oldname)
 	c.used[i] = c.used[i-1];
     c.used[0] = 0;
 
-#ifdef  DO_TAGGED_HASH
-    vtop = 0;
-    for (i = 0; i < NUSEDS; i++) {
-	if (vtop < c.vused[i])
-	    vtop = c.vused[i];
-	if (c.vused[i] == 0)
-	    newtable = 1;       /* hasn't got full usage history yet */
-    }
-    if (top != 0 && vtop == 0) {
-	DEBUG(("dbzagain: old table has no contents!\n"));
-	newtable = 1;
-    }
-    for (i = NUSEDS-1; i > 0; i--)
-	c.vused[i] = c.vused[i-1];
-    c.vused[0] = 0;
+    if (do_tagged_hash) {
+	vtop = 0;
+	for (i = 0; i < NUSEDS; i++) {
+	    if (vtop < c.vused[i])
+		vtop = c.vused[i];
+	    if (c.vused[i] == 0)
+		newtable = 1;   /* hasn't got full usage history yet */
+	}
+	if (top != 0 && vtop == 0) {
+	    DEBUG(("dbzagain: old table has no contents!\n"));
+	    newtable = 1;
+	}
+	for (i = NUSEDS-1; i > 0; i--)
+	    c.vused[i] = c.vused[i-1];
+	c.vused[0] = 0;
 
-    /* calculate tagging from old file */
-    if (stat(oldname, &sb) != -1 && vtop < sb.st_size)
-	vtop = sb.st_size;
-    config_by_text_size(&c, vtop);
-#endif
+	/* calculate tagging from old file */
+	if (stat(oldname, &sb) != -1 && vtop < sb.st_size)
+	    vtop = sb.st_size;
+	config_by_text_size(&c, vtop);
+    }
 
     newsize = dbzsize(top);
     if (!newtable || newsize > c.tsize) /* don't shrink new table */
@@ -679,15 +703,15 @@ BOOL dbzagain(const char *name, const char *oldname)
     }
 
     /* create and truncate .pag, or .index/.hash files */
-#ifdef  DO_TAGGED_HASH
-    if (!create_truncate(name, pag))
-	return FALSE;
-#else
-    if (!create_truncate(name, idx))
-	return FALSE;
-    if (!create_truncate(name, exists))
-	return FALSE;
-#endif
+    if (do_tagged_hash) {
+	if (!create_truncate(name, pag))
+	    return FALSE;
+    } else {
+	if (!create_truncate(name, idx))
+	    return FALSE;
+	if (!create_truncate(name, exists))
+	    return FALSE;
+    }
 
     /* and let dbzinit do the work */
     return dbzinit(name);
@@ -763,7 +787,7 @@ static void closehashtable(hash_table *tab) {
 #endif
 }
 
-#ifdef  DO_TAGGED_HASH
+/* for DO_TAGGED_HASH */
 static BOOL openbasefile(const char *name) {
     basef = fopen(name, "r");
     if (basef == NULL) {
@@ -785,7 +809,7 @@ static BOOL openbasefile(const char *name) {
 #endif
     return TRUE;
 }
-#endif  /* DO_TAGGED_HASH */
+/* end DO_TAGGED_HASH */
 
 /*
  * dbzinit - open a database, creating it (using defaults) if necessary
@@ -827,36 +851,32 @@ int dbzinit(const char *name) {
     }
 
     /* open pag or idx/exists file */
-#ifdef  DO_TAGGED_HASH
-    if (!openhashtable(name, pag, &pagtab, SOF, options.pag_incore)) {
-	fclose(dirf);
-	return FALSE;
+    if (do_tagged_hash) {
+	if (!openhashtable(name, pag, &pagtab, SOF, options.pag_incore)) {
+	    fclose(dirf);
+	    return FALSE;
+	}
+	if (!openbasefile(name)) {
+	    fclose(pagtab.f);
+	    close(pagtab.fd);
+	    fclose(dirf);
+	    return FALSE;
+	}
+	tagbits = conf.tagmask << conf.tagshift;
+	taghere = conf.tagenb << conf.tagshift;
+	tagboth = tagbits | taghere;
+    } else {
+	if (!openhashtable(name, idx, &idxtab, (cfg_extendeddbz ?
+		    sizeof(idxrecext) : sizeof(idxrec)), options.idx_incore)) {
+	    fclose(dirf);
+	    return FALSE;
+	}
+	if (!openhashtable(name, exists, &etab, sizeof(erec),
+		    options.exists_incore)) {
+	    fclose(dirf);
+	    return FALSE;
+	}
     }
-    if (!openbasefile(name)) {
-	fclose(pagtab.f);
-	close(pagtab.fd);
-	fclose(dirf);
-	return FALSE;
-    }
-    tagbits = conf.tagmask << conf.tagshift;
-    taghere = conf.tagenb << conf.tagshift;
-    tagboth = tagbits | taghere;
-#else
-    if ((innconf == NULL) && (ReadInnConf() < 0)) {
-	fclose(dirf);
-	return FALSE;
-    }
-    if (!openhashtable(name, idx, &idxtab, (innconf->extendeddbz ?
-		sizeof(idxrecext) : sizeof(idxrec)), options.idx_incore)) {
-	fclose(dirf);
-	return FALSE;
-    }
-    if (!openhashtable(name, exists, &etab, sizeof(erec),
-		options.exists_incore)) {
-	fclose(dirf);
-	return FALSE;
-    }
-#endif
 
     /* misc. setup */
     dirty = FALSE;
@@ -894,19 +914,19 @@ BOOL dbzclose(void)
     if (dbzsync() < 0)
 	ret = FALSE;
 
-#ifdef  DO_TAGGED_HASH
-    closehashtable(&pagtab);
-    if (fclose(basef) == EOF) {
-	fprintf(stderr, "dbzclose: fclose(basef) failed\n");
-	ret = FALSE;
+    if (do_tagged_hash) {
+	closehashtable(&pagtab);
+	if (fclose(basef) == EOF) {
+	    fprintf(stderr, "dbzclose: fclose(basef) failed\n");
+	    ret = FALSE;
+	}
+	if (basefname != NULL)
+	    free((POINTER)basefname);
+	basef = NULL;
+    } else {
+	closehashtable(&idxtab);
+	closehashtable(&etab);
     }
-    if (basefname != NULL)
-	free((POINTER)basefname);
-    basef = NULL;
-#else
-    closehashtable(&idxtab);
-    closehashtable(&etab);
-#endif
 
     if (fclose(dirf) == EOF) {
 	DEBUG(("dbzclose: fclose(dirf) failed\n"));
@@ -934,13 +954,14 @@ BOOL dbzsync(void)
 	return TRUE;;
 
     if (!options.writethrough) {
-#ifdef  DO_TAGGED_HASH
-	if (!putcore(&pagtab)) {
-#else
-	if (!putcore(&idxtab) || !putcore(&etab)) {
-#endif
-	    DEBUG(("dbzsync: putcore failed\n"));
-	    ret = FALSE;;
+	if (do_tagged_hash) {
+	    if (!putcore(&pagtab)) goto fail;
+	} else {
+	    if (!putcore(&idxtab) || !putcore(&etab)) {
+	      fail:
+		DEBUG(("dbzsync: putcore failed\n"));
+		ret = FALSE;;
+	    }
 	}
     }
 
@@ -951,7 +972,7 @@ BOOL dbzsync(void)
     return ret;
 }
 
-#ifdef  DO_TAGGED_HASH
+#if DO_TAGGED_HASH
 /*
  - okayvalue - check that a value can be stored
  */
@@ -964,23 +985,23 @@ static int okayvalue(of_t value) {
 #endif
     return(1);
 }
-#endif
+#endif /* DO_TAGGED_HASH */
 
 /* dbzexists - check if the given message-id is in the database */
 BOOL dbzexists(const HASH key) {
 
-#ifdef  DO_TAGGED_HASH
-    return (dbzfetch(key) != NOTFOUND);
+#if DO_TAGGED_HASH
+	return (dbzfetch(key) != NOTFOUND);
 #else
 	
-    if (!opendb) {
-	DEBUG(("dbzexists: database not open!\n"));
-	return FALSE;
-    }
+	if (!opendb) {
+	    DEBUG(("dbzexists: database not open!\n"));
+	    return FALSE;
+	}
 
-    prevp = FRESH;
-    start(&srch, key, FRESH);
-    return search(&srch);
+	prevp = FRESH;
+	start(&srch, key, FRESH);
+	return search(&srch);
 #endif
 }
 
@@ -1087,11 +1108,13 @@ BOOL dbzfetch(const HASH key, void *ivalue) {
     DEBUG(("fetch: found nothing\n"));  /*EK*/
     prevp = &srch;                      /* remember where we stopped */
     return NOTFOUND;
+
 #else   /* DO_TAGGED_HASH */
+
     if (search(&srch) == TRUE) {
 	/* Actually get the data now */
 	if ((options.idx_incore != INCORE_NO) && (srch.place < conf.tsize)) {
-	    if (innconf->extendeddbz)
+	    if (cfg_extendeddbz)
 		memcpy(ivalue, &((idxrecext *)idxtab.core)[srch.place], sizeof(idxrecext));
 	    else
 		memcpy(ivalue, &((idxrec *)idxtab.core)[srch.place], sizeof(idxrec));
@@ -1102,7 +1125,7 @@ BOOL dbzfetch(const HASH key, void *ivalue) {
 		srch.aborted = 1;
 		return FALSE;
 	    }
-	    if (innconf->extendeddbz) {
+	    if (cfg_extendeddbz) {
 		if (read(idxtab.fd, ivalue, sizeof(idxrecext)) != sizeof(idxrecext)) {
 		    DEBUG(("fetch: read failed\n"));
 		    idxtab.pos = -1;
@@ -1118,7 +1141,7 @@ BOOL dbzfetch(const HASH key, void *ivalue) {
 		}
 	    }
 	}
-	if (innconf->extendeddbz) {
+	if (cfg_extendeddbz) {
 	    iextvalue->offset[HISTOFFSET] = ntohl(iextvalue->offset[HISTOFFSET]);
 	    iextvalue->offset[OVEROFFSET] = ntohl(iextvalue->offset[OVEROFFSET]);
 	} else
@@ -1158,62 +1181,62 @@ DBZSTORE_RESULT dbzstore(const HASH key, void *ivalue) {
 	return DBZSTORE_ERROR;
     }
 
-#ifdef  DO_TAGGED_HASH
-    /* copy the value in to ensure alignment */
-    (void) memcpy((POINTER)&value, (POINTER)&data, SOF);
+#if DO_TAGGED_HASH
+	/* copy the value in to ensure alignment */
+	(void) memcpy((POINTER)&value, (POINTER)&data, SOF);
 
-    /* update maximum offset value if necessary */
-    if (value > conf.vused[0])
-	conf.vused[0] = value;
+	/* update maximum offset value if necessary */
+	if (value > conf.vused[0])
+	    conf.vused[0] = value;
 
-    /* now value is in fuzzy format */
-    value >>= conf.dropbits;
-    DEBUG(("dbzstore: (%ld)\n", (long)value));
+	/* now value is in fuzzy format */
+	value >>= conf.dropbits;
+	DEBUG(("dbzstore: (%ld)\n", (long)value));
 
-    if (!okayvalue(value)) {
-	DEBUG(("dbzstore: reserved bit or overflow in 0x%lx\n", value));
-	return DBZSTORE_ERROR;
-    }
+	if (!okayvalue(value)) {
+	    DEBUG(("dbzstore: reserved bit or overflow in 0x%lx\n", value));
+	    return DBZSTORE_ERROR;
+	}
 
-    /* find the place, exploiting previous search if possible */
-    start(&srch, key, prevp);
-    while (search(&srch) != NOTFOUND)
-	continue;
+	/* find the place, exploiting previous search if possible */
+	start(&srch, key, prevp);
+	while (search(&srch) != NOTFOUND)
+	    continue;
 
-    prevp = FRESH;
-    conf.used[0]++;
-    DEBUG(("store: used count %ld\n", conf.used[0]));
-    dirty = 1;
-    if (!set_pag(&srch, value))
-	return DBZSTORE_ERROR;
-    return DBZSTORE_OK;
-#else   /* DO_TAGGED_HASH */
+	prevp = FRESH;
+	conf.used[0]++;
+	DEBUG(("store: used count %ld\n", conf.used[0]));
+	dirty = 1;
+	if (!set_pag(&srch, value))
+	    return DBZSTORE_ERROR;
+	return DBZSTORE_OK;
+#else        /* do_tagged_hash */
 
-    /* find the place, exploiting previous search if possible */
-    start(&srch, key, prevp);
-    if (search(&srch) == TRUE)
-	return DBZSTORE_EXISTS;
+	/* find the place, exploiting previous search if possible */
+	start(&srch, key, prevp);
+	if (search(&srch) == TRUE)
+	    return DBZSTORE_EXISTS;
 
-    prevp = FRESH;
-    conf.used[0]++;
-    DEBUG(("store: used count %ld\n", conf.used[0]));
-    dirty = TRUE;
+	prevp = FRESH;
+	conf.used[0]++;
+	DEBUG(("store: used count %ld\n", conf.used[0]));
+	dirty = TRUE;
 
-    if (innconf->extendeddbz) {
-	iextvalue->offset[HISTOFFSET] = htonl(iextvalue->offset[HISTOFFSET]);
-	iextvalue->offset[OVEROFFSET] = htonl(iextvalue->offset[OVEROFFSET]);
-    } else
-	ionevalue->offset = htonl(ionevalue->offset);
-    memcpy(&evalue.hash, &srch.hash,
-	   sizeof(evalue.hash) < sizeof(srch.hash) ? sizeof(evalue.hash) : sizeof(srch.hash));
+	if (cfg_extendeddbz) {
+	    iextvalue->offset[HISTOFFSET] = htonl(iextvalue->offset[HISTOFFSET]);
+	    iextvalue->offset[OVEROFFSET] = htonl(iextvalue->offset[OVEROFFSET]);
+	} else
+	    ionevalue->offset = htonl(ionevalue->offset);
+	memcpy(&evalue.hash, &srch.hash,
+	       sizeof(evalue.hash) < sizeof(srch.hash) ? sizeof(evalue.hash) : sizeof(srch.hash));
 
-    /* Set the value in the index first since we don't care if it's out of date */
-    if (!set(&srch, &idxtab, ivalue))
-	return DBZSTORE_ERROR;
-    if (!set(&srch, &etab, &evalue))
-	return DBZSTORE_ERROR;
-    return DBZSTORE_OK;
-#endif  /* DO_TAGGED_HASH */
+	/* Set the value in the index first since we don't care if it's out of date */
+	if (!set(&srch, &idxtab, ivalue))
+	    return DBZSTORE_ERROR;
+	if (!set(&srch, &etab, &evalue))
+	    return DBZSTORE_ERROR;
+	return DBZSTORE_OK;
+#endif
 }
 
 /*
@@ -1226,60 +1249,61 @@ static BOOL getconf(FILE *df, dbzconfig *cp) {
     int i;
 
     /* empty file, no configuration known */
-#ifdef  DO_TAGGED_HASH
-    if (df == NULL) {
-	cp->tsize = DEFSIZE;
-	for (i = 0; i < NUSEDS; i++)
-	    cp->used[i] = 0;
-	for (i = 0; i < NUSEDS; i++)
-	    cp->vused[i] = 0;
-	cp->valuesize = sizeof(OFFSET_T);
-	cp->fillpercent = 50;
-	cp->tagenb = TAGENB;
-	cp->tagmask = TAGMASK;
-	cp->tagshift = TAGSHIFT;
-	cp->dropbits = 0;
-	cp->lenfuzzy = 0;
-	DEBUG(("getconf: defaults (%ld, (0x%lx/0x%lx<<%d %d))\n",
-				   cp->tsize, cp->tagenb,
-				   cp->tagmask, cp->tagshift, cp->dropbits));
-	return TRUE;
-    }
+    if (do_tagged_hash) {
+	if (df == NULL) {
+	    cp->tsize = DEFSIZE;
+	    for (i = 0; i < NUSEDS; i++)
+		cp->used[i] = 0;
+	    for (i = 0; i < NUSEDS; i++)
+		cp->vused[i] = 0;
+	    cp->valuesize = sizeof(OFFSET_T);
+	    cp->fillpercent = 50;
+	    cp->tagenb = TAGENB;
+	    cp->tagmask = TAGMASK;
+	    cp->tagshift = TAGSHIFT;
+	    cp->dropbits = 0;
+	    cp->lenfuzzy = 0;
+	    DEBUG(("getconf: defaults (%ld, (0x%lx/0x%lx<<%d %d))\n",
+				       cp->tsize, cp->tagenb,
+				       cp->tagmask, cp->tagshift, cp->dropbits));
+	    return TRUE;
+	}
 
-    i = fscanf(df, "dbz 6 %ld %d %d %ld %ld %d %d\n", &cp->tsize,
-		&cp->valuesize, &cp->fillpercent, &cp->tagenb,
-		&cp->tagmask, &cp->tagshift, &cp->dropbits);
-    if (i != 7) {
-	DEBUG(("getconf error"));
-	return FALSE;
-    }
-    if (cp->valuesize != sizeof(OFFSET_T)) {
-	DEBUG(("getconf: wrong of_t size (%d)\n", cp->valuesize));
-	return FALSE;
-    }
-    cp->lenfuzzy = (int)(1 << cp->dropbits) - 1;
-#else   /* DO_TAGGED_HASH */
-    if (df == NULL) {
-	cp->tsize = DEFSIZE;
-	for (i = 0; i < NUSEDS; i++)
-	    cp->used[i] = 0;
-	cp->valuesize = sizeof(idxrec) + sizeof(erec);
-	cp->fillpercent = 66;
-	DEBUG(("getconf: defaults (%ld)\n", cp->tsize));
-	return TRUE;
-    }
+	i = fscanf(df, "dbz 6 %ld %d %d %ld %ld %d %d\n", &cp->tsize,
+		    &cp->valuesize, &cp->fillpercent, &cp->tagenb,
+		    &cp->tagmask, &cp->tagshift, &cp->dropbits);
+	if (i != 7) {
+	    DEBUG(("getconf error"));
+	    return FALSE;
+	}
+	if (cp->valuesize != sizeof(OFFSET_T)) {
+	    DEBUG(("getconf: wrong of_t size (%d)\n", cp->valuesize));
+	    return FALSE;
+	}
+	cp->lenfuzzy = (int)(1 << cp->dropbits) - 1;
+    } else { /* do_tagged_hash */
+	if (df == NULL) {
+	    cp->tsize = DEFSIZE;
+	    for (i = 0; i < NUSEDS; i++)
+		cp->used[i] = 0;
+	    cp->valuesize = sizeof(idxrec) + sizeof(erec);
+	    cp->fillpercent = 66;
+	    DEBUG(("getconf: defaults (%ld)\n", cp->tsize));
+	    return TRUE;
+	}
+
+	i = fscanf(df, "dbz 6 %ld %d %d\n", &cp->tsize,
+		    &cp->valuesize, &cp->fillpercent);
+	if (i != 3) {
+	    DEBUG(("getconf error"));
+	    return FALSE;
+	}
+	if (cp->valuesize != (sizeof(idxrec) + sizeof(erec))) {
+	    DEBUG(("getconf: wrong of_t size (%d)\n", cp->valuesize));
+	    return FALSE;
+	}
+    } /* do_tagged_hash */
 
-    i = fscanf(df, "dbz 6 %ld %d %d\n", &cp->tsize,
-		&cp->valuesize, &cp->fillpercent);
-    if (i != 3) {
-	DEBUG(("getconf error"));
-	return FALSE;
-    }
-    if (cp->valuesize != (sizeof(idxrec) + sizeof(erec))) {
-	DEBUG(("getconf: wrong of_t size (%d)\n", cp->valuesize));
-	return FALSE;
-    }
-#endif  /* DO_TAGGED_HASH */
     DEBUG(("size %ld\n", cp->tsize));
 
     /* second line, the usages */
@@ -1290,15 +1314,15 @@ static BOOL getconf(FILE *df, dbzconfig *cp) {
 	}
     DEBUG(("used %ld %ld %ld...\n", cp->used[0], cp->used[1], cp->used[2]));
 
-#ifdef  DO_TAGGED_HASH
-    /* third line, the text usages */
-    for (i = 0; i < NUSEDS; i++)
-	if (!fscanf(df, "%ld", &cp->vused[i])) {
-	    DEBUG(("getconf error\n"));
-	    return FALSE;
-	}
-    DEBUG(("vused %ld %ld %ld...\n", cp->vused[0], cp->vused[1], cp->vused[2]));
-#endif  /* DO_TAGGED_HASH */
+    if (do_tagged_hash) {
+	/* third line, the text usages */
+	for (i = 0; i < NUSEDS; i++)
+	    if (!fscanf(df, "%ld", &cp->vused[i])) {
+		DEBUG(("getconf error\n"));
+		return FALSE;
+	    }
+	DEBUG(("vused %ld %ld %ld...\n", cp->vused[0], cp->vused[1], cp->vused[2]));
+    }
 
     return TRUE;
 }
@@ -1315,22 +1339,22 @@ static int putconf(FILE *f, dbzconfig *cp) {
 	ret = -1;
     }
 
-#ifdef  DO_TAGGED_HASH
-    fprintf(f, "dbz %d %ld %d %d %ld %ld %d %d\n", dbzversion, cp->tsize,
-		cp->valuesize, cp->fillpercent, cp->tagenb,
-		cp->tagmask, cp->tagshift, cp->dropbits);
-#else   /* DO_TAGGED_HASH */
-    fprintf(f, "dbz %d %ld %d %d\n", dbzversion, cp->tsize,
-		   cp->valuesize, cp->fillpercent);
-#endif  /* DO_TAGGED_HASH */
+    if (do_tagged_hash) {
+	fprintf(f, "dbz %d %ld %d %d %ld %ld %d %d\n", dbzversion, cp->tsize,
+		    cp->valuesize, cp->fillpercent, cp->tagenb,
+		    cp->tagmask, cp->tagshift, cp->dropbits);
+    } else {
+	fprintf(f, "dbz %d %ld %d %d\n", dbzversion, cp->tsize,
+		    cp->valuesize, cp->fillpercent);
+    }
 
     for (i = 0; i < NUSEDS; i++)
 	fprintf(f, "%ld%c", cp->used[i], (i < NUSEDS-1) ? ' ' : '\n');
 
-#ifdef  DO_TAGGED_HASH
-    for (i = 0; i < NUSEDS; i++)
-	fprintf(f, "%ld%c", cp->vused[i], (i < NUSEDS-1) ? ' ' : '\n');
-#endif
+    if (do_tagged_hash) {
+	for (i = 0; i < NUSEDS; i++)
+	    fprintf(f, "%ld%c", cp->vused[i], (i < NUSEDS-1) ? ' ' : '\n');
+    }
 
     fflush(f);
     if (ferror(f))
@@ -1752,15 +1776,15 @@ int timediffms(struct timeval start, struct timeval end) {
 void RemoveDBZ(char *filename) {
     char fn[1024];
 
-#ifdef  DO_TAGGED_HASH
-    sprintf(fn, "%s.pag", filename);
-    unlink(fn);
-#else
-    sprintf(fn, "%s.exists", filename);
-    unlink(fn);
-    sprintf(fn, "%s.index", filename);
-    unlink(fn);
-#endif
+    if (do_tagged_hash) {
+	sprintf(fn, "%s.pag", filename);
+	unlink(fn);
+    } else {
+	sprintf(fn, "%s.exists", filename);
+	unlink(fn);
+	sprintf(fn, "%s.index", filename);
+	unlink(fn);
+    }
     sprintf(fn, "%s.dir", filename);
     unlink(fn);
 }
@@ -1872,7 +1896,7 @@ char *argv[];
 		exit(1);
 	    }
 #else
-	    if (innconf->extendeddbz) {
+	    if (cfg_extendeddbz) {
 		if (!dbzfetch(key, &iextvalue)) {
 		    fprintf(stderr, "line %d can't fetch %s\n", line, ibuf);
 		    exit(1);
