@@ -1,13 +1,15 @@
-/*  $Id: dbz.h,v 1.3 1998/07/12 09:39:19 src Exp $
+/*  $Id: dbz.h,v 1.4 1999/03/05 11:20:33 src Exp $
+ *
+ *  for dbz version 6
  *
  *  $Log: dbz.h,v $
+ *  Revision 1.4  1999/03/05 11:20:33  src
+ *  Delayed decision of dbz v3 vz. 6 to runtime.
+ *
  *  Revision 1.3  1998/07/12 09:39:19  src
  *  NewsX version 1.0
- *
  */
 
-#if DBZ_VERSION==6
-
 /*
  *  v6
  *
@@ -19,14 +21,3 @@
 #include "libinn.h"
 
 #include "dbz-v6.h"
-
-#else
-
-/* 
- *  v3
- */
-#include "dbz-v3.h"
-
-#endif
-
-
