@@ -1,4 +1,4 @@
-/*  VER 110  TAB P   $Id: active.c,v 1.9 1998/09/09 07:32:10 src Exp $
+/*  VER 117  TAB P   $Id: active.c,v 1.12 1999/03/16 08:12:46 src Exp $
  *
  *  handle the local active file
  *
@@ -6,6 +6,15 @@
  *  the GNU General Public License applies
  *
  *  $Log: active.c,v $
+ *  Revision 1.12  1999/03/16 08:12:46  src
+ *  Moved clean_active() to active.c
+ *
+ *  Revision 1.11  1999/03/07 14:58:17  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.10  1998/10/25 13:43:07  src
+ *  Minor code readability issue
+ *
  *  Revision 1.9  1998/09/09 07:32:10  src
  *  Version 1.1
  *
@@ -26,6 +35,10 @@
 #include "proto.h"
 #include "options.h"
 #include "news.h"
+#include "newsconfig.h"
+
+static void
+clear_active(void);
 
 /* 
  *  globals for active 
@@ -69,10 +82,8 @@ active_group(char *group)
     /* BUG: check if already? */
     n = sizeof(ACTIVE_GRP) + strlen(group);
 
-    if (!(ap = malloc(n))) {
-	log_msg(L_ERRno,"out of memory");
-	exit_cleanup(1);
-    }
+    ap = xmalloc(n);
+
     /* add to list of active groups at that hashindex */
     ap->seen = 0;
     strcpy(ap->name,group);
@@ -86,7 +97,7 @@ active_group(char *group)
 int 
 is_in_active(char *group)
 {
-    return find_active(group) ? 1 : 0;
+    return find_active(group) != 0;
 }
 
 /*
@@ -139,14 +150,12 @@ unseen_active(char *group)
 /*
  *  load local active file
  *  assumes load_sys has been performed
- *  return true if more
  */
 void
-load_active(void)
+load_active(int again)
 {
     FILE *f;
     int n;
-    char activename[PATH_MAX];
     char buf[BUFSIZ];
     char *p;
     char group[BUFSIZ];
@@ -156,13 +165,12 @@ load_active(void)
 
     progtitle("read active");
 
-    build_alt_filename(activename,news_home(),ACTIVE,active);
-
-    if (!(f = fopen(activename,"r"))) {
-	log_msg(L_ERRno,"can't open \"%s\"",activename);
+    if (!(f = fopen(cfg_active,"r"))) {
+	log_msg(L_ERRno,"can't open \"%s\"",cfg_active);
 	exit_cleanup(8);
     }
-    log_msg(L_DEBUG,"reading %s",activename);
+    log_msg(L_DEBUG,"%sreading %s",(again ? "re-":""), cfg_active);
+    if (again) clear_active();
 
     /* load local active file */
     while (fgets(buf,BUFSIZ,f)) {
@@ -171,7 +179,7 @@ load_active(void)
 	    while (isspace(*p)) ++p;
 	    if (p[0] && p[0] != '#') {
 		n = sscanf(p,"%[^ \n\t!:] %ld %ld %c",group,&a,&b,&isact);
-		if (!bad_syntax(buf,n,4,activename)) {
+		if (!bad_syntax(buf,n,4,cfg_active)) {
 		    if (isact == 'y' || isact == 'm') {
 			/* group is active */
 			active_group(group);
@@ -188,14 +196,20 @@ load_active(void)
 
 /*
  *  clear local active file
- *  NOTE: looses allocated memory
  */
-void
+static void
 clear_active(void)
 {
     int h;
+    ACTIVE_GRP *p,*pn;
 
     for (h=0; h<ACTIVE_HASH; ++h) {
+	p = activegrp[h];
+	while (p) {
+	    pn = p->next;
+	    free(p);
+	    p = pn;
+	}
 	activegrp[h] = 0;
     }
 }
