@@ -1,4 +1,4 @@
-/*  VER 201   TAB P   $Id: doit.c,v 1.12 1998/09/11 16:37:43 src Exp $
+/*  VER 211   TAB P   $Id: doit.c,v 1.18 1999/04/05 19:01:51 src Exp $
  *
  *  an NNTP news exchange client
  *
@@ -6,6 +6,24 @@
  *  the GNU General Public License applies
  *
  *  $Log: doit.c,v $
+ *  Revision 1.18  1999/04/05 19:01:51  src
+ *  Lock for scanlogs
+ *
+ *  Revision 1.17  1999/03/16 08:12:46  src
+ *  Moved clean_active() to active.c
+ *
+ *  Revision 1.16  1999/03/16 08:03:30  src
+ *  Renamed lock() to do_lock()
+ *
+ *  Revision 1.15  1999/03/11 07:30:00  src
+ *  Implemented check for spool free space
+ *
+ *  Revision 1.14  1998/11/22 08:00:07  src
+ *  Option --no-queue
+ *
+ *  Revision 1.13  1998/10/21 07:31:22  src
+ *  Opened syslog at an an earlier stage
+ *
  *  Revision 1.12  1998/09/11 16:37:43  src
  *  Lockfile for logfile of posted articles and for posted article folder.
  *
@@ -25,6 +43,7 @@
 #include "common.h"
 #include "proto.h"
 #include "options.h"
+#include "newsconfig.h"
 #include "nntp.h" 
 #include "news.h"
 
@@ -64,6 +83,27 @@ doit(void)
 	return 0;
     }
 
+    /*
+     *  wait in case scanlogs is running 
+     *  NOTE: this probably only makes sense if scanlogs is patched to
+     *        read newsx logs
+     */
+    if (inn_opt && scanlogs_lock) {
+	int d = 0;
+	char lockpath[PATH_MAX];
+
+	progtitle("check scanlogs running");
+	build_filename(lockpath,LOCKS,"/","LOCK.scanlogs",NULL);
+	while (check_lock(lockpath)) {
+	    if (d==0) progtitle("await scanlogs lock");
+	    if ((d += LOCKDELTA) > SCANLOGTIME) {
+		log_msg(L_ERR,"scanlog lock \"%s\" timed out", lockpath);
+		return 13;
+	    }
+	    sleep(LOCKDELTA);
+	}
+    }
+
     if (!no_host_lock) {
 	/* 
 	 *  lock other newsxes accessing same host out first
@@ -72,11 +112,9 @@ doit(void)
 
 	progtitle("locking host access");
 	build_filename(lockname,LOCK_PREFIX,hostname,NULL,NULL);
-	lock(NULL,lockname);
+	do_lock(NULL,lockname,noqueue);
     }
 
-    log_open();
-
     time(&starttime);
     log_msg(L_DEBUG,"newsx %s %s", VERSION, text_time(starttime));
     log_msg(L_DEBUG,"server: %s, spool: %s", hostname, spoolname);
@@ -84,7 +122,7 @@ doit(void)
     /* load the active file before we connect to the server */
     if (!nopull_opt || group_newlist || group_desc) {
 	load_sys();
-	load_active();
+	load_active(0);
     }
 
     /* see if there are any articles batched up */
@@ -115,7 +153,8 @@ doit(void)
     /* start the timer now */
     time(&starttime);
 
-    /* We might need to do MODE READER before AUTHINFO or afterwards.
+    /*
+     * we might need to do MODE READER before AUTHINFO or afterwards.
      * readbeforeauth_opt switches which way round we do things. -- PMM
      */
 
@@ -135,8 +174,7 @@ doit(void)
 	get_list();
 	if (!nopull_opt || group_desc) {
 	    /* the active list may have been updated by now... */
-	    clear_active();
-	    load_active();
+	    load_active(1);
 	}
     }
 
@@ -155,6 +193,34 @@ doit(void)
 
     /* and pull any new articles */
     if (!nopull_opt) {
+	/*
+	 *  check spool free space first
+	 */
+	if (minfree) {
+	    long kbyte, inode;
+
+	    /* BUG: what about cfg_incoming? */
+	    log_msg(L_DEBUG,"check spool free space");
+	    if (!freespace(cfg_spool, &kbyte, &inode)) {
+		log_msg(errno ? L_ERRno:L_ERR,
+			"can't determine free %s space",cfg_spool);
+	    } else {
+		/* BUG: what about inodes? */
+		if (kbyte < minfree) {
+		    log_msg(L_ERR,"spool free space is %ldk too low",
+							minfree-kbyte);
+		    goto no_fetch;
+		}
+		/*
+		 *  NOTE: assume avg. 2k per article
+		 */
+		if (inode < minfree/2) {
+		    log_msg(L_ERR,"spool free inodes is %ldk too low",
+							minfree/2 - inode);
+		}
+	    }
+	}
+
 	log_msg(L_DEBUG,"fetching news");
 	if (!pull(spoolname)) {
 	    /*
@@ -168,6 +234,7 @@ doit(void)
 	    return 9;
 	}
     }
+    no_fetch:
 
     /* time for goodbye */
     if (!noaction_opt)
