@@ -1,4 +1,4 @@
-/*  VER 061  TAB P   $Id: exec.c,v 1.7 1998/09/09 07:32:11 src Exp $
+/*  VER 070  TAB P   $Id: exec.c,v 1.12 1999/04/07 21:07:59 src Exp $
  *
  *  run a script
  *
@@ -6,6 +6,21 @@
  *  the GNU General Public License applies
  *
  *  $Log: exec.c,v $
+ *  Revision 1.12  1999/04/07 21:07:59  src
+ *  Debugged config support
+ *
+ *  Revision 1.11  1999/04/07 20:09:16  src
+ *  Implemented configuration reading, and eval mechanism
+ *
+ *  Revision 1.10  1999/03/31 03:38:28  src
+ *  Moved errno.h to common.h
+ *
+ *  Revision 1.9  1999/03/07 14:58:18  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.8  1998/11/21 19:14:23  src
+ *  Added --filter option
+ *
  *  Revision 1.7  1998/09/09 07:32:11  src
  *  Version 1.1
  *
@@ -20,9 +35,9 @@
 #include "proto.h"
 #include "options.h"
 #include "news.h"
+#include "newsconfig.h"
 
 #include <fcntl.h>
-#include <errno.h>
 #if HAVE_SYS_WAIT_H
 #  include <sys/wait.h>
 #endif
@@ -99,7 +114,7 @@ ctlinnd(char *arg1, char *arg2)
      */
     sprintf(t_opt,"-t%ld",timeout ? timeout : LOCKTIME);
 	
-    build_alt_filename(ctlpath,NEWSBIN,INN_CTL,ctlpgm);
+    build_alt_filename(ctlpath,cfg_newsbin,INN_CTL,ctlpgm);
 
     if ((pid=do_fork()) == 0) {
 	/* slave */
@@ -115,6 +130,7 @@ ctlinnd(char *arg1, char *arg2)
 
 /*
  *  run a program with files in and out plumbed as stdin and stdout
+ *  return program status
  */
 int 
 script(char *program, int in, int out)
@@ -130,6 +146,7 @@ script(char *program, int in, int out)
 	if (err >= 0) dup2(err, STDERR);
 	setuid(getuid());
 	setgid(getgid());
+	/* BUG: configure this... */
 	execl("/bin/sh", "sh", "-c", program, (char *)0);
 	log_msg(L_ERRno,"could not exec /bin/sh");
 	exit_cleanup(1);
@@ -167,6 +184,7 @@ program_open(char *program, SOCKET_D *sock)
 	if (err >= 0) dup2(err, STDERR);
 	setuid(getuid());
 	setgid(getgid());
+	/* BUG: config /bin/sh */
 	execl("/bin/sh", "sh", "-c", program, (char *)0);
 	log_msg(L_ERRno,"could not exec /bin/sh");
 	exit_cleanup(1);
@@ -210,7 +228,7 @@ fork_write(char *path,int *pidp,char *arg1,char *arg2)
 	dup2(pipe_fd[0],STDIN); /* connect stdin to the pipe */
 	close(pipe_fd[1]);
 	
-	log_msg(L_DEBUG,"exec: %s %s %s",path,arg1?arg1:"",
+	log_msg(L_DEBUG,"exec: %s args: %s %s",path,arg1?arg1:"",
 						  arg2?arg2:"");
 	execl(path,path,arg1,arg2,NULL);
 
@@ -222,7 +240,48 @@ fork_write(char *path,int *pidp,char *arg1,char *arg2)
     /* parent process */
     close(pipe_fd[0]);
     if (!(f = fdopen(pipe_fd[1],"w"))) {
-	log_msg(L_ERRno,"can't connect pipe: %s",path);
+	log_msg(L_ERRno,"can't connect write pipe: %s",path);
+	return 0;
+    }
+    *pidp = pid;
+    return f;
+}
+
+/*
+ *  fork a process
+ *  open up a stream to read from it
+ */
+FILE *
+fork_read(char *path,int *pidp,char *arg)
+{ 
+    FILE *f;
+    pid_t pid;
+    int pipe_fd[2];
+
+    /* normal mode */
+    if (pipe(pipe_fd) == -1) {
+	log_msg(L_ERRno,"can't pipe: %s",path);
+	return 0;
+    }
+	
+    if (!(pid = fork())) {
+	/* child */
+	close(STDOUT);      /* disconnect stdout */
+	dup2(pipe_fd[1],STDOUT); /* connect stdout to the pipe */
+	close(pipe_fd[0]);
+	
+	log_msg(L_DEBUG,"exec: %s arg: %s",path,arg?arg:"");
+	execl(path,path,arg,NULL,NULL);
+
+	/* here on exec failure: reject with "system down" status */
+	log_msg(L_ERRno,"can't exec: %s",path);
+	return 0;
+    }
+
+    /* parent process */
+    close(pipe_fd[1]);
+    if (!(f = fdopen(pipe_fd[0],"r"))) {
+	log_msg(L_ERRno,"can't connect read pipe: %s",path);
 	return 0;
     }
     *pidp = pid;
