@@ -1,4 +1,4 @@
-/*  VER 200   TAB P   $Id: getarticle.c,v 1.12 1998/09/21 10:04:29 src Exp $
+/*  VER 228   TAB P   $Id: getarticle.c,v 1.26 1999/04/18 09:28:45 src Exp $
  *
  *  fetch articles via an NNTP server
  *
@@ -10,6 +10,48 @@
  *  the GNU General Public License applies
  *
  *  $Log: getarticle.c,v $
+ *  Revision 1.26  1999/04/18 09:28:45  src
+ *  Fix for --tag with --no-path
+ *
+ *  Revision 1.25  1999/04/07 07:10:55  src
+ *  Implemented --tag
+ *
+ *  Revision 1.24  1999/03/31 05:53:45  src
+ *  Seperated MAXHEADERSIZE from NNTP_STRLEN
+ *
+ *  Revision 1.23  1999/03/24 03:52:58  src
+ *  Implemented "newsx" as magic exclude pattern
+ *
+ *  Revision 1.22  1999/03/18 05:41:19  src
+ *  Version 1.4pre
+ *
+ *  Revision 1.21  1999/03/16 17:35:38  src
+ *  Version 1.4pre
+ *
+ *  Revision 1.20  1999/03/15 08:51:35  src
+ *  Nothing.
+ *
+ *  Revision 1.19  1999/03/11 07:30:00  src
+ *  Implemented check for spool free space
+ *
+ *  Revision 1.18  1999/03/07 14:58:18  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.17  1999/03/04 17:19:27  src
+ *  Fix for removed groups that weren't
+ *
+ *  Revision 1.16  1998/11/22 08:23:07  src
+ *  Added --forget-inactive
+ *
+ *  Revision 1.15  1998/11/21 19:14:23  src
+ *  Added --filter option
+ *
+ *  Revision 1.14  1998/10/25 13:50:10  src
+ *  STAT fix from Steinar Haug, sthaug@nethelp.no
+ *
+ *  Revision 1.13  1998/10/23 03:37:42  src
+ *  Path line error report fixed, reported by Uli Zappe
+ *
  *  Revision 1.12  1998/09/21 10:04:29  src
  *  Added new command line options for --inews
  *
@@ -32,6 +74,8 @@
 #include "proto.h"
 #include "options.h"
 #include "statistics.h"
+#include "newsconfig.h"
+#include "news.h"
 #include "nntp.h"
 
 long bytes_in_spool; /* external: number of bytes read */
@@ -51,35 +95,46 @@ char *temp_store = 0;
 long temp_len = 0;
 #define TEMP_STEP 100000 /* BUG: tuneable? */
 
-static char cur_group[80]; /* for error reporting */
 static int no_stat = 0; /* for hosts that lack a STAT */
 
+/*
+ *  see if message-ID is fake
+ */
+static int
+is_fake(char *msgid)
+{
+    while (*msgid && isspace(*msgid)) ++msgid;
+    return is_tag(msgid,"<fake@");
+}
+
 /*
  *  read an article proper
  *  and feed it to the spool
- *  argument is just for error reports
+ *  arguments are just for error reports
  *  return false on errors that mean we should not continue
  */
 static int 
-read_article(long where)
+read_article(long where,char *group)
 {
-    char line[NNTP_STRLEN+1];
+    char line[MAXHEADERSIZE+1];
+    char path_line[MAXHEADERSIZE+1];
+    int reject = 0;
     int newline = 1;
     int header = 1;
-    int ispath = 0;
     int path_ok = -1;
-    char *path_line = 0;
     long len;
     long bytecount = 0L; /* BUG: */
 
-    line[NNTP_STRLEN] = '\0'; /* better safe than sorry */
+    line[MAXHEADERSIZE] = '\0'; /* better safe than sorry */
+    path_line[0] = '\0';
 
     /* fetch the article, header and body */
     for (;;) {
-	if (!get_server_msg(line, NNTP_STRLEN)) {
+	if (!get_server_msg(line, MAXHEADERSIZE)) {
 	    /* timeout: simply give up */
 	    return 0;
 	}
+	if (filter) filter_line(line); /* send to filter too */
 	len = strlen(line);
 	gross_bytecount += len;
 
@@ -93,23 +148,57 @@ read_article(long where)
 	    case '\n':
 		header = 0;
 		break;
+	    case 'M':         /* 12345678901 */
+		if (is_tag(line,"Message-ID:")) {
+		    if (!keep_fake && is_fake(line+11)) {
+			/* Dnews style faked message */
+			reject = 1;
+		    }
+		}
+		break;
 	    case 'P':
 		if (is_tag(line,"Path:")) {
+		    char *path = skipsp(line+5);
+		    int add_tag = tag_opt;
 		    /* BUG: what about veeeery long lines - point into buffer instead */
 		    /* BUG: or can there really be long lines? */
-		    path_ok = traverse_path(path_line = line+5);
-		    ispath = 1;
+
+		    /* fix by Stefan Huelswitt <huels@iname.com> */
+		    /* save for error reporting et al. */
+		    strcpy(path_line,path);
+
+		    if (nopath_opt) {
+			/* ignore path check */
+			path_ok = 1;
+		    } else if ((path_ok = traverse_path(path)) != 1) {
+			/* path did not match */
+
+			if (newsx_path) {
+			    add_tag = 1;
+			}
+		    }
+
+		    if (add_tag) {
+			if (len < MAXHEADERSIZE-(2+strlen(NEWSX_PATH))) {
+			/* save the day by inserting "magic" path */
+			    sprintf(line,"Path: %s!%s",NEWSX_PATH,
+							path_line);
+			    len = strlen(line);
+
+			    log_msg(L_DEBUGMORE,"inserting %s path",
+								NEWSX_PATH);
+			    if (newsx_path) path_ok = 1;
+			} else {
+			    log_msg(L_ERR,
+			    "No room to extend path in ARTICLE %ld, adjust MAXHEADERSIZE",
+								where);
+			    reject = -1;
+			}
+		    }
 		    break;
 		}
-	    default:
-		ispath = 0;
 		break;
-	    case ' ':
-	    case '\t':
-		if (ispath && (line[0]==' ' || line[0]=='\t')) {
-		    /* BUG: can path really be on a continue line? */
-		    path_ok |= traverse_path(line+1);
-		}
+	    default:
 		break;
 	    }
 	}
@@ -138,25 +227,43 @@ read_article(long where)
 	if (newline) temp_store[bytecount++] = '\n';
     }
 
-    /* see if the path is suspect */
-    if (!nopath_opt && path_ok != 1) {
-	if (path_line) {
-	    /* specified Path is wrong */
-	    log_msg(L_ERR,"ARTICLE %ld did not contain '%s' in 'Path:'", 
-					where, get_exclusion());
-	    log_msg(L_ERR,"Path line was '%s'", path_line);
-	    /* BUG: here and elsewhere: say which newsgroup, for Pete's sake... */
-	} else {
-	    /* no Path specified */
-	    if (!add_header || strncmp(add_header,"Path",4)!=0) {
-		log_msg(L_ERR,"ARTICLE %ld did not contain a 'Path:'", where);
-	    }
+    /* reject article? */
+    if (reject) {
+	if (reject > 0) {
+	    log_msg(L_DEBUG,"ARTICLE %ld rejected as fake",where);
+	    ++fake_articles;
 	}
 	return 0;
     }
 
- /* sprintf(line," writing %ld bytes", bytecount); */
- /* progtitle2(cur_group, line); */
+    /* get status of filtering */
+    if (filter && !reject) {
+	/* get filter status */
+	char id[20];
+	sprintf(id,"%ld",where);
+	if (filter_reject(id)) {
+	    log_msg(L_DEBUG,"ARTICLE %ld rejected by full filter",where);
+	    ++filter_articles;
+	    return 0;
+	}
+    }
+
+    /* see if the path is suspect */
+    if (!nopath_opt && path_ok != 1) {
+	if (path_line[0]) {
+	    /* specified Path is wrong */
+	    log_msg(L_ERR,"ARTICLE %ld in %s did not contain '%s' in 'Path:'",
+				   where, group,            get_exclusion());
+	    log_msg(L_ERR,"Path line was '%s'", path_line);
+	} else {
+	    /* no Path specified */
+	    if (!add_header || strncmp(add_header,"Path",4)!=0) {
+		log_msg(L_ERR,"ARTICLE %ld in %s did not contain a 'Path:'", 
+				       where, group);
+	    }
+	}
+	return 0;
+    }
 
     if (!write_incoming(temp_store,bytecount,(path_ok >= 0))) return 0;
 
@@ -172,7 +279,7 @@ read_article(long where)
  *  return false if no point in continuing
  */
 static int 
-handle_article(char *status, long where)
+handle_article(char *status, long where, char *group)
 {
     int ok;
     long a;
@@ -192,11 +299,12 @@ handle_article(char *status, long where)
 		    told = 1;
 		}
 	    } else {
-		log_msg(L_ERR,"ARTICLE %ld out of phase: %s", where, status);
+		log_msg(L_ERR,"ARTICLE %ld in %s out of phase: %s",
+					where, group, status);
 		return 0;
 	    }
 	}
-	if ((ok = read_article(where))) {
+	if ((ok = read_article(where,group))) {
 	    /* article is presumably OK */
 	    latest_where = where;
 
@@ -209,13 +317,12 @@ handle_article(char *status, long where)
     case ERR_NOART:                     /* no such article */
 	/* BUG: message does not contain an article number... */
 	/* article has disappeared, ignore it */
-	log_msg(L_DEBUG,"article in %s disappeared: %s",cur_group,status);
+	log_msg(L_DEBUG,"article in %s disappeared: %s",group,status);
 	latest_where = where;
 
 	return 1;
 
     default:                            /* otherwise, protocol error */
-     /* progtitle2(cur_group, ", error"); */
 	log_msg(L_ERR,"NNTP article read error: got \"%s\"", status);
 	/* stop here */
 	return 0;
@@ -227,18 +334,18 @@ handle_article(char *status, long where)
  *  return false if no point in continuing
  */
 static int 
-current_article(long where)
+current_article(long where,char *group)
 {
     char request[NNTP_STRLEN+1];
 
     if (no_stat) {
 	sprintf(request, " %ld", where);
-	progtitle2(cur_group, request);
+	progtitle2(group, request);
     }
 
     sprintf(request, "ARTICLE %ld%s", where, newline);
     latest_contains_msgnum = 1;
-    return put_request(request,handle_article,where);
+    return put_request(request,handle_article,where,group);
 }
 
 /*
@@ -246,10 +353,18 @@ current_article(long where)
  *  either by doing nothing, or by requesting an ARTICLE
  */
 static int 
-process_msgid(char *msgid,long where)
+process_msgid(char *msgid,long where, char *group)
 {
+    /* check if fake message */
+    if (!keep_fake && is_fake(msgid)) {
+	log_msg(L_DEBUGMORE,"MSGID %s is fake",msgid);
+	++fake_articles;
+	latest_where = where;
+	return 1;
+    }
+
     /* check if already in news history database */
-    if ((!history || history[0]) && history_lookup(msgid)) {
+    if ((!cfg_history || cfg_history[0]) && history_lookup(msgid)) {
 	++history_articles;
 	latest_where = where;
 	return 1;
@@ -262,21 +377,28 @@ process_msgid(char *msgid,long where)
     }
     /* check if triggered by message ID filter */
     if (do_mfilter(msgid)) {
-	++mfilter_articles;
+	log_msg(L_DEBUGMORE,"MSGID %s filtered by message ID",msgid);
+	++filter_articles;
+	latest_where = where;
+	return 1;
+    }
+    if (do_filter(msgid,group)) {
+	log_msg(L_DEBUGMORE,"MSGID %s filtered by header",msgid);
+	++filter_articles;
 	latest_where = where;
 	return 1;
     }
     /* BUG: have max file size?? */
 
     log_msg(L_DEBUGMORE,"fetching article %ld",where);
-    return current_article(where);
+    return current_article(where,group);
 }
 
 /*
  *  handle article STAT result  
  */
 static int 
-handle_stat(char *status,long where)
+handle_stat(char *status,long where,char *group)
 {
     char msgid[NNTP_STRLEN+1];
     long a;
@@ -295,26 +417,24 @@ handle_stat(char *status,long where)
 	}
 	stat_failures = 0;
 
-	return process_msgid(msgid,a);
+	return process_msgid(msgid,a,group);
 
     case ERR_COMMAND:                   /* STAT is not implemented */
-     /* progtitle2(cur_group, ", no STAT"); */
 	log_msg(L_INFO,"server lacks STAT command: %s",status);
 	no_stat = 1;
     case 99: /* dummy if no STAT command */
 	log_msg(L_DEBUGMORE,"unconditionally fetching article %ld",where);
-	return current_article(where);
+	return current_article(where,group);
 
     case ERR_NOARTIG:                   /* no such article in group */
     case ERR_NOART:                     /* no such article */
-     /* progtitle2(cur_group, ", no article"); */
 	/* article no longer there, ignore it */
 	++stat_failures;
-	log_msg(L_DEBUGMORE,"article %ld in %s not on server",where,cur_group);
+	log_msg(L_DEBUGMORE,"article %ld in %s not on server, stat_failures %d",
+				     where,group,stat_failures);
 	return 1;
 
     default:                            /* otherwise, protocol error */
-     /* progtitle2(cur_group, ", STAT error"); */
 	log_msg(L_ERR,"NNTP unknown STAT error: got \"%s\"", status);
 	/* stop here */
 	return 0;
@@ -325,7 +445,7 @@ handle_stat(char *status,long where)
  *  handle article NEXT result
  */
 static int 
-handle_next(char *status,long where)
+handle_next(char *status,long where,char *group)
 {
     char msgid[NNTP_STRLEN+1];
     long a;
@@ -342,21 +462,21 @@ handle_next(char *status,long where)
 	    no_next = 1;
 	    sprintf(msgid, "STAT %ld%s", where,newline);
 	    latest_contains_msgnum = 1;
-	    return put_request(msgid,handle_stat,where);
+	    return put_request(msgid,handle_stat,where,group);
 	}
 	latest_next = a;
-	return process_msgid(msgid,a);
+	return process_msgid(msgid,a,group);
 
     case ERR_COMMAND:                   /* NEXT is not implemented */
 	log_msg(L_INFO,"server lacks NEXT command: %s",status);
 	no_next = 1;
 	log_msg(L_DEBUGMORE,"unconditionally fetching article %ld",where);
-	return current_article(where);
+	return current_article(where,group);
 
     case ERR_NONEXT:                    /* no next article */
 	/* article no longer there, ignore it */
 	next_at_witts_end = 1;
-	log_msg(L_DEBUGMORE,"NEXT reports no more articles in %s",cur_group);
+	log_msg(L_DEBUGMORE,"NEXT reports no more articles in %s",group);
 	return 1;
 
     case ERR_NOARTIG:                   /* no such article in group */
@@ -374,10 +494,11 @@ handle_next(char *status,long where)
 
 /*
  *  fetch an article in current group
+ *  the group name is just for error reporting and filtering
  *  return false if no point in continuing
  */
 int 
-fetch_article(long where,int first)
+fetch_article(long where,int first,char *group)
 {
     char request[NNTP_STRLEN+1];
     char status[NNTP_STRLEN+1];
@@ -397,12 +518,12 @@ fetch_article(long where,int first)
     if (no_stat) {
 	sprintf(status,"%d",99);
 	/* fake dummy STAT message that will cause an ARTICLE */
-	return handle_stat(status,where);
+	return handle_stat(status,where,group);
     }
 
     /* enquire article status and message ID */
     sprintf(request, " %ld", where);
-    progtitle2(cur_group, request);
+    progtitle2(group, request);
 
     if (!first && !no_next && stat_failures >= stat_attempts) {
 	if (next_at_witts_end) {
@@ -429,17 +550,21 @@ fetch_article(long where,int first)
 
 	    if (n < 0) break; /* queue is empty */
 	}
+	if (stat_failures > 0) {
+	    log_msg(L_DEBUGMORE,"before NEXT: stat_failures %d", stat_failures);
+	    goto issue_a_stat;
+	}
 	sprintf(request, "NEXT%s", newline);
 	++next_requests;
 	latest_contains_msgnum = 0;
-	return put_request(request,handle_next,where);
+	return put_request(request,handle_next,where,group);
     }
 
     /* issue a standard STAT */
   issue_a_stat:
     sprintf(request, "STAT %ld%s", where,newline);
     latest_contains_msgnum = 1;
-    return put_request(request,handle_stat,where);
+    return put_request(request,handle_stat,where,group);
 }
 
 /*
@@ -447,7 +572,7 @@ fetch_article(long where,int first)
  *  return 1 if OK, 0 if no group, -1 if no point in continuing
  */
 static int 
-handle_group(char *status,char *group,long *firstp,long *lastp)
+select_a_group(char *status,char *group,long *firstp,long *lastp)
 {
     long msgs;
     char *endptr;
@@ -485,8 +610,7 @@ select_group(char *group,long *firstp,long *lastp)
     char request[NNTP_STRLEN+1];
     char status[NNTP_STRLEN+1];
 
-    strncpy(cur_group,group,sizeof(cur_group)-1); /* for error reporting */
-    progtitle(cur_group);
+    progtitle(group);
 
     sprintf(request, "GROUP %s%s", group,newline);
 
@@ -495,6 +619,6 @@ select_group(char *group,long *firstp,long *lastp)
     /* get status */
     if (!get_server_nntp(status, sizeof(status))) return -1;
 
-    return handle_group(status,group,firstp,lastp);
+    return select_a_group(status,group,firstp,lastp);
 }
 
