@@ -1,4 +1,4 @@
-/*  VER 057  TAB P   $Id: history.c,v 1.10 1998/09/14 05:27:34 src Exp $
+/*  VER 071  TAB P   $Id: history.c,v 1.19 1999/04/08 10:56:13 src Exp $
  *  
  *  history database lookup
  *
@@ -6,6 +6,33 @@
  *  the GNU General Public License applies
  *
  *  $Log: history.c,v $
+ *  Revision 1.19  1999/04/08 10:56:13  src
+ *  *** empty log message ***
+ *
+ *  Revision 1.18  1999/04/08 10:48:54  src
+ *  Extendeddbz to configure
+ *
+ *  Revision 1.17  1999/04/08 10:19:37  src
+ *  Check for extendeddbz in configure
+ *
+ *  Revision 1.16  1999/03/31 03:38:28  src
+ *  Moved errno.h to common.h
+ *
+ *  Revision 1.15  1999/03/09 11:43:57  src
+ *  Cosmetic fixes
+ *
+ *  Revision 1.14  1999/03/07 15:40:58  src
+ *  Removed DBZ_VERSION from configuration
+ *
+ *  Revision 1.13  1999/03/07 14:58:18  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.12  1999/03/05 11:20:42  src
+ *  Delayed decision of dbz v3 vz. 6 to runtime.
+ *
+ *  Revision 1.11  1999/03/04 16:20:20  src
+ *  Renamed dbz v3, fixed hostname in spool
+ *
  *  Revision 1.10  1998/09/14 05:27:34  src
  *  Declared offset
  *
@@ -22,37 +49,25 @@
 #include "common.h"
 #include "proto.h"
 #include "options.h"
+#include "newsconfig.h"
 #include "news.h"
 
-#include <errno.h>
-
 #if WITH_HISTORY
 
 #ifdef HAVE_LIBDBZ
+    /* BUG: this is pretty archaic */
 #  include <dbz.h>
-#  undef HAVE_LIBDBM
 #else
-#  ifdef HAVE_LIBDBM
-#    include <dbm.h>
-#  else
-#    include "../dbz/dbz.h"
-#  endif
+#  include "../dbz/dbz.h"
 #endif
 
-#ifndef HAVE_LIBDBM
-#if DBZ_VERSION==6
-#  define dbminit   dbzinit
-#  define dbmclose  dbzclose
-#else
-#  define dbminit   dbzdbminit
-#  define fetch     dbzdbmfetch
-#  define dbmclose  dbzdbmclose
-#endif
-#endif
+#include "../dbz/dbz-v3.h"
 
-/* negative if error (or disable) */
+/* negative if error (or disabled) */
 int hist_open = 0;
 
+static int dbz_vers = 0;
+
 /*
  *  look up history database
  *  will convert message ID to news character case conventions
@@ -61,23 +76,35 @@ int hist_open = 0;
 int 
 history_lookup(char *msgid)
 {
+    int result;
+
     if (hist_open <= 0) {
 	if (hist_open == 0) {
-	    char historyname[PATH_MAX];
-	
-	    build_alt_filename(historyname,news_home(),HISTORY,history);
 
-	    if (dbminit(historyname) == -1) {
+	    /* determine dbz version */
+	    if (dbz_vers == 0) {
+		dbz_vers = dbz_version(cfg_history);
+		/* BUG: what about errors? */
+	    }
+
+	    if (dbz_vers == 3) {
+		/* regardless of HAVE_LIBDBZ */
+		result = dbzv3_dbminit(cfg_history);
+	    } else {
+		result = dbzinit(cfg_history);
+	    }
+	    if (result == -1) {
 		if (errno == EDOM)
-		    log_msg(L_ERR,"database \"%s\" in wrong format",historyname);
+		    log_msg(L_ERR,"database \"%s\" in wrong format",cfg_history);
 		else 
-		    log_msg(L_ERRno,"can't open database \"%s\"",historyname);
+		    log_msg(L_ERRno,"can't open database \"%s\"",cfg_history);
 		hist_open = -1;
 		return 0;
 	    }
-	    log_msg(L_DEBUGMORE,"history file %s opened",historyname);
+	    log_msg(L_DEBUGMORE,"history file %s opened",cfg_history);
 	    hist_open = 1;
 	} else {
+	    /* history file failed before, so don't bother */
 	    return 0;
 	}
     } 
@@ -85,16 +112,42 @@ history_lookup(char *msgid)
     /* 
      *  look up database 
      */
-#if DBZ_VERSION==6
-    {
+    if (dbz_vers == 3) {
+	/*
+	 *  dbz v.3
+	 */
+	dbzv3_datum key;
+	dbzv3_datum val;
+	char *p;
+	int len = strlen(msgid);
+
+	/* convert host name to lower case */
+	if ((p = strchr(msgid,'@'))) {
+	    while (*++p) *p = tolower(*p);
+	}
+
+	key.dptr = msgid;
+	key.dsize = len+1;
+	
+	val = dbzv3_dbmfetch(key);
+	if (!(val.dptr)) {
+	    log_msg(L_DEBUGMORE,"msgid %s not in history",msgid);
+	    return 0;
+	}
+    } else {
+	/*
+	 *  dbz v.6 et.al.
+	 */
 	HASH key = HashMessageID(msgid);
+	OFFSET_T offset;
 #  ifdef DO_TAGGED_HASH
-	OFFSET_T offset = dbzfetch(key);
+	offset = dbzfetch(key);
 #  else
 	idxrec ionevalue;
 	idxrecext iextvalue;
-	int offset = 0;
-	if (innconf->extendeddbz) {
+	offset = 0;
+
+	if (cfg_extendeddbz) {
 	    if (!dbzfetch(key, &iextvalue))
 		offset = -1;
 	    /* offset = iextvalue.offset[HISTOFFSET]; */
@@ -109,31 +162,6 @@ history_lookup(char *msgid)
 	    return 0;
 	}
     }
-#else
-    {
-	/*
-	 *  dbz v.3
-	 */
-	datum key;
-	datum val;
-	char *p;
-	int len = strlen(msgid);
-
-	/* convert host name to lower case */
-	if ((p = strchr(msgid,'@'))) {
-	    while (*++p) *p = tolower(*p);
-	}
-
-	key.dptr = msgid;
-	key.dsize = len+1;
-	
-	val = fetch(key);
-	if (!(val.dptr)) {
-	    log_msg(L_DEBUGMORE,"msgid %s not in history",msgid);
-	    return 0;
-	}
-    }
-#endif
     /* OK, we found it */
     return 1;
 }
@@ -145,7 +173,11 @@ void
 history_done(void)
 {
     if (hist_open > 0) {
-	dbmclose();
+	if (dbz_vers == 3) {
+	    dbzv3_dbmclose();
+	} else {
+	    dbzclose();
+	}
 	hist_open = 0; 
     }
 }
