@@ -1,4 +1,4 @@
-/*  VER 035  TAB P   $Id: incoming.c,v 1.9 1998/09/27 07:34:33 src Exp $
+/*  VER 043  TAB P   $Id: incoming.c,v 1.16 1999/04/07 07:10:55 src Exp $
  *
  *  place incoming news in spool
  *
@@ -6,6 +6,27 @@
  *  the GNU General Public License applies
  *
  *  $Log: incoming.c,v $
+ *  Revision 1.16  1999/04/07 07:10:55  src
+ *  Implemented --tag
+ *
+ *  Revision 1.15  1999/04/06 03:43:40  src
+ *  INN hostname for incoming spool
+ *
+ *  Revision 1.14  1999/04/03 12:13:44  src
+ *  Fixed error in mktemp usage, reported by Stefan Huelswitt <huels@iname.com>
+ *
+ *  Revision 1.13  1999/03/31 05:53:46  src
+ *  Seperated MAXHEADERSIZE from NNTP_STRLEN
+ *
+ *  Revision 1.12  1999/03/14 08:47:54  src
+ *  Uprated log for innreport.
+ *
+ *  Revision 1.11  1999/03/07 14:58:18  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.10  1999/03/04 16:20:20  src
+ *  Renamed dbz v3, fixed hostname in spool
+ *
  *  Revision 1.9  1998/09/27 07:34:33  src
  *  Removed snprintf
  *
@@ -25,7 +46,9 @@
 #include "common.h"
 #include "proto.h"
 #include "options.h"
+#include "newsconfig.h"
 #include "news.h"
+#include "self.h"
 #include "nntp.h"
 
 /* 
@@ -37,15 +60,6 @@ static char intmpname[PATH_MAX];
 static FILE *pull_spoolfp = 0; /* currently open spool */
 static int pull_pid = 0; 
 
-/*
- *  get name of (incoming) spool directory
- */
-static void
-incoming_spool(char *name /* size is PATH_MAX */)
-{
-    build_alt_filename(name,SPOOL,INCOMING,incoming);
-}
-
 /*
  *  open spool file
  */
@@ -59,14 +73,11 @@ open_spool(void)
 	    }
 	} else {
 	    /* temporary name */
-	    char incomingdir[PATH_MAX];
 	    char pid_string[100];
-	    int pid = getpid();
-	
-	    sprintf(pid_string,".%d",pid); /* add PID to temporary filename */
 
-	    incoming_spool(incomingdir);
-	    build_filename(intmpname,incomingdir,_IN_TMP,spoolname,pid_string);
+	    sprintf(pid_string,".%d",my_pid); /* add PID to temporary filename */
+
+	    build_filename(intmpname,cfg_incoming,_IN_TMP,spoolname,pid_string);
 	
 	    /* open temporary spool file proper */
 	    if (!(pull_spoolfp = fopen(intmpname,"w"))) {
@@ -86,7 +97,7 @@ int
 write_incoming(char *buffer,long bytecount,int is_a_path)
 {
     char *extra_header = NULL;
-    char buf[2*NNTP_STRLEN+1];
+    char path_buf[2*NNTP_STRLEN+1];
 
     if (bytecount > 0) {
 	/* open outgoing spool */
@@ -99,8 +110,8 @@ write_incoming(char *buffer,long bytecount,int is_a_path)
 		if (!is_a_path) {
 		    char *p = get_exclusion();
 		    if (!*p) p = hostname;
-		    sprintf(buf,"Path: %s!not-for-mail",p);
-		    extra_header = buf;
+		    sprintf(path_buf,"Path: %s!not-for-mail",p);
+		    extra_header = path_buf;
 		}
 	    } else {
 		extra_header = add_header;
@@ -155,9 +166,6 @@ flush_spool(int urgent)
 	    }
 	    pull_spoolfp = 0;
 	} else {
-	    char incomingdir[PATH_MAX];
-	    char spoolfile[PATH_MAX];
-	
 	    progtitle("pull: flush spool");
 	    log_msg(l_debug,"flush spool");
 	    f = pull_spoolfp;
@@ -169,10 +177,25 @@ flush_spool(int urgent)
 	
 	    /* spool file is now ready for newsrun */
 	    /* rename it so that it becomes visible */
-	    sprintf(spoolfile,_INSPOOL,time((time_t *)0),getpid(),++sequence);
+	    if (inn_opt && isalpha(hostname[0])) {
+		/*
+		 *  INN style batch name:
+		 *  <hostname><XXXXXX>
+		 */
+		build_filename(inspoolname,cfg_incoming,"/",spoolname,_INN_SPOOL);
+		if (!mktemp(inspoolname)) goto old_style;
+	    } else {
+		char spoolfile[PATH_MAX];
 	
-	    build_alt_filename(incomingdir,SPOOL,INCOMING,incoming);
-	    build_filename(inspoolname,incomingdir,"/",spoolfile,NULL);
+		/*
+		 *  Cnews style:
+		 *  <digit><unique>.t
+		 */
+	      old_style:
+		sprintf(spoolfile,_C_SPOOL,(long)time((time_t *)0),
+					    (int)my_pid, ++sequence);
+		build_filename(inspoolname,cfg_incoming,"/",spoolfile,NULL);
+	    }
 
 	    log_msg(l_debug,"rename spool from %s to %s",
 						intmpname,inspoolname);
