@@ -1,4 +1,4 @@
-/*  VER 113   TAB P   $Id: lock.c,v 1.11 1998/09/11 16:37:43 src Exp $
+/*  VER 137   TAB P   $Id: lock.c,v 1.23 1999/04/09 04:25:30 src Exp $
  *
  *  lock file handling 
  *
@@ -30,6 +30,42 @@
  *  we don't need the general lock
  *
  *  $Log: lock.c,v $
+ *  Revision 1.23  1999/04/09 04:25:30  src
+ *  Scanlogs: no error message if no lockfile; mea culpa
+ *
+ *  Revision 1.22  1999/04/08 10:19:37  src
+ *  Check for extendeddbz in configure
+ *
+ *  Revision 1.21  1999/04/05 19:01:51  src
+ *  Lock for scanlogs
+ *
+ *  Revision 1.20  1999/03/31 03:38:28  src
+ *  Moved errno.h to common.h
+ *
+ *  Revision 1.19  1999/03/19 07:44:36  src
+ *  Fixed bug re. leftover lockfiles introduced in 1.4pre
+ *
+ *  Revision 1.18  1999/03/16 08:25:44  src
+ *  Fixed bug re. multiple locks.
+ *
+ *  Revision 1.17  1999/03/16 08:03:30  src
+ *  Renamed lock() to do_lock()
+ *
+ *  Revision 1.16  1999/03/14 08:47:54  src
+ *  Uprated log for innreport.
+ *
+ *  Revision 1.15  1999/03/07 14:58:18  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.14  1998/11/22 10:48:22  src
+ *  Let --no-queue check for stale lockfiles
+ *
+ *  Revision 1.13  1998/11/22 08:00:08  src
+ *  Option --no-queue
+ *
+ *  Revision 1.12  1998/10/25 04:08:56  src
+ *  A little more debugging on lockfiles.
+ *
  *  Revision 1.11  1998/09/11 16:37:43  src
  *  Lockfile for logfile of posted articles and for posted article folder.
  *
@@ -52,19 +88,25 @@
 #include "proto.h"
 #include "options.h"
 #include "stat.h"
+#include "self.h"
+#include "newsconfig.h"
 
+#include <fcntl.h>
 #include <signal.h>
-#include <errno.h>
 
 /*
  *  local
  */
 static int 
-force_unlock(char *name,int sig);
+force_unlock(char *name,int sig,int giveup);
 static int
 lockfile_exists(char *name);
 static int
 add_to_lockfiles(char *name);
+static void
+drop_lock(int id);
+static pid_t
+pid_of_lock(char* name,int should_exist);
 
 /*
  *  lockfiles
@@ -77,22 +119,23 @@ static char *lock_file_list[MAX_LOCKS];
  *  return lock ID
  */
 int
-lock(char *dir_name /* NULL if in main lock */, char *lock_name)
+do_lock(char *dir_name /* NULL if in main lock */,
+	char *lock_name,
+	int giveup)
 {
     long t = 0L;
     long locktime = timeout ? timeout : LOCKTIME;
-    int pid;
-    char pid_buf[20];
+    char pid_buf[50];
     int sig = SIGUSR1; /* start as gentle as possible */
     FILE *f;
     static char name_temp[PATH_MAX];
     static char name_lock[PATH_MAX];
     int id = -1;
+    int id_temp = -1;
 
-    pid = getpid();
-    sprintf(pid_buf,"%d",pid);
+    sprintf(pid_buf,"%d",my_pid);
 
-    if (locks && strcmp(locks,"none")==0) {
+    if (!cfg_locks || strcmp(cfg_locks,"none")==0) {
 	log_msg(L_DEBUG,"no lock");
 	return -1;
     }
@@ -107,21 +150,18 @@ lock(char *dir_name /* NULL if in main lock */, char *lock_name)
 	build_filename(name_temp,name_lock,".",pid_buf,NULL);
     } else {
 	/* common lock directory */
-	char locks_dir[PATH_MAX];
-	build_alt_filename(locks_dir,NEWSHOME,LOCKS,locks);
-
-	build_filename(name_lock,locks_dir,"/",lock_name,NULL);
+	build_filename(name_lock,cfg_locks,"/",lock_name,NULL);
 	/* BUG: the C News convention is used for the temp_name */
-	build_filename(name_temp,locks_dir,"/L.",pid_buf,NULL);
+	build_filename(name_temp,cfg_locks,"/L.",pid_buf,NULL);
     }
 
     if (!lockfile_exists(name_temp)) {
-	/* build our own little file */
+	/* build our own little file (temporary name) */
 	if (!(f = fopen(name_temp,"w"))) {
 	    log_msg(L_ERRno,"can't create lock \"%s\"", name_temp);
 	    exit_cleanup(5);
 	}
-	id = add_to_lockfiles(name_temp);
+	id_temp = add_to_lockfiles(name_temp);
 
 	fprintf(f,"%s\n",pid_buf);
 	if (fclose(f) == EOF) {
@@ -130,19 +170,19 @@ lock(char *dir_name /* NULL if in main lock */, char *lock_name)
 	}
     }
 
-    /* perform the locking action */
-    for (;;) {
-	/* BUG: check if name_temp has disappeared... */
+    /* perform the locking action by linking */
+    while (link(name_temp,name_lock) < 0) {
+	/*
+	 *  there was an error
+	 */
+
 	/* BUG: what if interrupt within link()? */
-	if (link(name_temp,name_lock) >= 0) {
-	    /* locked successfully */
-	    add_to_lockfiles(name_lock);
+	/* BUG: check if name_temp has disappeared? */
 
-	    log_msg(L_DEBUGMORE,"created lock: %s",name_lock);
-	    break;
-	} 
+	log_msg(L_DEBUGMORE,"waiting: link %s %s failed: %s",
+				name_temp,name_lock,str_error(errno));
 
-	/* failed, tried force? */
+	/* failed, tried using force? */
 	if (sig == SIGKILL) {
 	    log_msg(L_ERR,"can't lock \"%s\" after forced unlock", name_lock);
 	    exit_cleanup(5);
@@ -160,87 +200,78 @@ lock(char *dir_name /* NULL if in main lock */, char *lock_name)
 	    else 
 		log_msg(L_ERR,"file lock \"%s\" timed out", name_lock);
 
-	    /* file remains locked, what shall we do? */
-	    if (force_unlock(name_lock,sig)) {
-		/* we've probably managed to remove the lock by force */
-		sig = (sig==SIGUSR1) ? SIGTERM : SIGKILL;
+	    /* there is a lockfile, what shall we do? */
+	    if (force_unlock(name_lock,sig,giveup)) {
+		/* signal worked */
+		if (sig==SIGUSR1) {
+		    /* the other process is known to exist */
+		    /* wait, then use persuasion if lockfile persists */
+		    sig = SIGTERM; 
+		} else {
+		    /* we've presumably managed to remove the lock */
+		    /* use real force if lockfile persists */
+		    sig = SIGKILL; 
+		}
 	    } else {
 		/* give up */
-		exit_cleanup(5);
+		exit_cleanup(13);
 	    }
 	}
 	sleep(LOCKDELTA);
     }
+
+    /* locked successfully - update our tables */
+    id = add_to_lockfiles(name_lock);
+
+    log_msg(L_DEBUGMORE,"created lock: %s",name_lock);
+
     return id;
 }
 
 /*
- *  lock failed, try to force a unlock
+ *  lock failed, try to send a signal to find out if the other
+ *  process exists (SIGUSR1) or to force an unlock (SIGTERM/SIGKILL)
  *  return true on success
  *  BUG: we really should have checked that it is the 
  *       same process that has been locking all the time
  */
 static int 
-force_unlock(char *name,int sig)
+force_unlock(char *name,int sig,int giveup)
 {
 #ifdef NO_FORCE
     log_msg(L_INFO,"unlock by force permanently disabled");
     return 0;
 #else
     char name_other[PATH_MAX];
-    char *p;
-    char pid_buf[100];
-    int other_pid = -1;
+    pid_t other_pid;
     struct stat st;
-    FILE *f;
 
-    if (noforce_opt) {
+    if (sig != SIGUSR1 && noforce_opt) {
 	log_msg(L_INFO,"no attempt at unlocking by force");
 	return 0;
     }
 
     /* read the file to find the pid */
-    if (!(f = fopen(name,"r"))) {
-	log_msg(L_ERRno,"lockfile \"%s\" unreadable", name);
-	/* assume file has disappeared */
-	return 1;
-    }
-    pid_buf[0] = '\0';
-    fgets(pid_buf,sizeof(pid_buf),f);
-    fclose(f);
 
-    for (p=pid_buf; isspace(*p); ++p)
-	;
-    if (!isdigit(*p)) {
+    if (!(other_pid = pid_of_lock(name,1))) {
 	/* 
 	 *  whatever shall we do? give up?
 	 *  we assume the risk, and continue
 	 */ 
-	log_msg(L_ERRno,"lockfile \"%s\" has wrong format",
-			     name);
-	return 0;
-    } 
-    if ((other_pid = atoi(p)) < 0) {
-	/* 
-	 *  can't allow this, so again, we continue
-	 */ 
-	log_msg(L_ERRno,"lockfile \"%s\" has negative pid",
-			     name);
-
 	return 0;
     } 
 
     /* contents is plausible */
-    if (other_pid == getpid()) {
+    if (other_pid == my_pid) {
 	/* 
 	 *  ourselves? shouldn't happen 
 	 */
-	log_msg(L_ERRno,"file \"%s\" locked by ourselves?", name);
+	log_msg(L_ERR,"file \"%s\" locked by ourselves?", name);
 	other_pid = -1;
     } else {
 	/*     
 	 *  OK, we know who it is
-	 *  we try to execute a pretty sure kill
+	 *  we try to execute a kill according to sig
 	 */
 	log_msg(L_INFO,"sending process %d signal %d", other_pid, sig);
 
@@ -250,13 +281,20 @@ force_unlock(char *name,int sig)
 		log_msg(L_INFO,"process %d does not exist", other_pid);
 	    } else {
 		/* we probably don't have permission for this */
-		log_msg(L_ERRno,"kill stale process %s failed", pid_buf);
+		log_msg(L_ERRno,"kill stale process %ld failed", 
+							(long)other_pid);
 		return 0;
 	    }
 	} else {
 	    if (sig==SIGUSR1) {
-		log_msg(L_INFO,"process %d exists, wait...", other_pid);
-		return 1;
+		if (giveup) {
+		    log_msg(L_ERR,"process %d has locked \"%s\", don't queue up",
+					   other_pid,   name);
+		    return 0; /* exit */
+		} else {
+		    log_msg(L_INFO,"process %d exists, wait...", other_pid);
+		    return 1;
+		}
 	    }
 	}
 	/* give the process time to clean up */
@@ -276,6 +314,8 @@ force_unlock(char *name,int sig)
 	
     /* close the other processes private lockfile too */
     if (other_pid != -1) {
+	char pid_buf[50];
+	sprintf(pid_buf,"%d",other_pid);
 	build_filename(name_other,name,".",pid_buf,NULL);
 	if (unlink(name_other) == -1) {
 	    if (stat(name_other,&st) != -1) {
@@ -290,7 +330,7 @@ force_unlock(char *name,int sig)
 
 /*
  *  add to lockfile list
- *  should be reasonably interruptable
+ *  should be reasonably interruptable (!)
  */
 static int
 add_to_lockfiles(char *name)
@@ -335,14 +375,26 @@ unlock_one(int id)
 
     if (id < 0 || id >= MAX_LOCKS) return;
 
-    if (lock_file_list[id]) {
-	p = lock_file_list[id];
-	lock_file_list[id] = NULL;
+    if ((p = lock_file_list[id])) {
 	if (unlink(p) == -1)
 	    log_msg(L_ERRno,"unlink lock file failed: %s", p);
 	else
 	    log_msg(L_DEBUGMORE,"unlocked: %s", p);
-	free(p);
+	drop_lock(id);
+    }
+}     
+
+/*
+ *  drop a lock file from the list
+ */
+static void
+drop_lock(int id)
+{
+    if (id < 0 || id >= MAX_LOCKS) return;
+
+    if (lock_file_list[id]) {
+	free(lock_file_list[id]);
+	lock_file_list[id] = NULL;
     }
 }     
 
@@ -365,9 +417,63 @@ unlock_all(void)
 void 
 unlock_exit(int n)
 {
-    progtitle("exit");
+    progtitle("unlock/exit");
     history_done();
+    close_article();
     unlock_all();
     exit(n);
 }
 
+/*
+ *  see if the process named in an existing lock still exists by
+ *  sending it a null signal (adapted from INN/shlock.c)
+ */
+int
+check_lock(char* name)
+{
+    pid_t pid;
+
+    if (!(pid = pid_of_lock(name,0))) return 0;
+
+    /* send the signal */
+    if (kill(pid, 0) < 0 && errno == ESRCH) return 0;
+
+    /* either the kill worked, or we're optimistic about the error code */
+    return 1;
+}
+
+/*
+ *  get pid belonging to lock file
+ *  return 0 if problems
+ */
+static pid_t
+pid_of_lock(char* name,int should_exist)
+{
+    int fd;
+    pid_t pid;
+    int i;
+    char buff[BUFSIZ];
+
+    /* open the file */
+    if ((fd = open(name, O_RDONLY)) < 0) {
+	if (should_exist) log_msg(L_ERRno,"can't access lockfile \"%s\"",name);
+	return 0;
+    }
+
+    /* read the PID that is written there. */
+    if ((i = read(fd, buff, sizeof buff - 1)) <= 0) {
+	log_msg(L_ERRno,"lockfile \"%s\" is empty", name);
+	close(fd);
+	return 0;
+    }
+    buff[i] = '\0';
+    pid = (pid_t) atol(buff);
+
+    close(fd);
+    if (pid <= 0) {
+	log_msg(L_ERR,"lockfile \"%s\" has wrong format", name);
+	return 0;
+    }
+
+    return pid;
+}
