@@ -1,4 +1,4 @@
-/*  VER 077  TAB P   $Id: logmsg.c,v 1.8 1998/09/09 07:32:12 src Exp $
+/*  VER 079  TAB P   $Id: logmsg.c,v 1.10 1999/03/31 03:38:28 src Exp $
  *
  *  handle error messages and such...
  *
@@ -6,6 +6,12 @@
  *  the GNU General Public License applies
  *
  *  $Log: logmsg.c,v $
+ *  Revision 1.10  1999/03/31 03:38:28  src
+ *  Moved errno.h to common.h
+ *
+ *  Revision 1.9  1998/10/25 04:08:56  src
+ *  A little more debugging on lockfiles.
+ *
  *  Revision 1.8  1998/09/09 07:32:12  src
  *  Version 1.1
  *
@@ -20,8 +26,6 @@
 #include "proto.h"
 #include "options.h"
 
-#include <errno.h>
-
 #if WITHOUT_ERRLOG
 #  undef HAVE_SYSLOG_H
 #endif
@@ -107,6 +111,20 @@ clean_line(void)
     }
 }
 
+/*
+ *  make error message from error number
+ */
+char *
+str_error(int err)
+{
+#if HAVE_STRERROR
+    return strerror(err);
+#else
+    static char buf[30];
+    sprintf("errno %d",err);
+#endif
+}
+
 /*
  *  log a message   
  *
@@ -155,11 +173,9 @@ log_msg(int type, char *fmt, char *arg1,char *arg2,char *arg3,char *arg4)
     case L_ERR:
 	e = errno;
 	vsprintf(buf, fmt, ap);
-#if HAVE_STRERROR
 	if (type == L_ERRno) {
-	    sprintf(buf + strlen (buf), ": %s", strerror(e));
+	    sprintf(buf + strlen (buf), ": %s", str_error(e));
 	}
-#endif
 	strcat(buf, "\n");
 #if HAVE_SYSLOG_H
 	if (!debug_opt) {
