@@ -1,4 +1,4 @@
-/*  VER 246   TAB P   $Id: main.c,v 1.18 1998/09/21 10:04:29 src Exp $
+/*  VER 302   TAB P   $Id: main.c,v 1.46 1999/04/25 07:00:37 src Exp $
  *
  *  an NNTP news exchange client
  *
@@ -20,6 +20,90 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  *  $Log: main.c,v $
+ *  Revision 1.46  1999/04/25 07:00:37  src
+ *  Mellowed error message for missing standard profiles
+ *
+ *  Revision 1.45  1999/04/19 04:20:07  src
+ *  Profile for specific spool
+ *
+ *  Revision 1.44  1999/04/19 03:51:03  src
+ *  Added config options for newsq
+ *
+ *  Revision 1.43  1999/04/19 03:32:38  src
+ *  Standard profile in $PATHETC/newsx.conf
+ *
+ *  Revision 1.42  1999/04/17 15:00:52  src
+ *  Documentation for 1.4
+ *
+ *  Revision 1.41  1999/04/08 07:31:29  src
+ *  Support for a list of runtime configuration files.
+ *
+ *  Revision 1.40  1999/04/08 06:31:09  src
+ *  Support for full runtime configuration
+ *
+ *  Revision 1.39  1999/04/07 20:09:16  src
+ *  Implemented configuration reading, and eval mechanism
+ *
+ *  Revision 1.38  1999/04/07 08:02:11  src
+ *  Implemented --profile
+ *
+ *  Revision 1.37  1999/04/07 07:46:46  src
+ *  Changes longopts slightly
+ *
+ *  Revision 1.36  1999/04/07 07:10:55  src
+ *  Implemented --tag
+ *
+ *  Revision 1.35  1999/04/05 19:01:51  src
+ *  Lock for scanlogs
+ *
+ *  Revision 1.34  1999/03/30 17:43:56  src
+ *  Got rid of "subscript has type char" warning message
+ *
+ *  Revision 1.33  1999/03/16 17:35:38  src
+ *  Version 1.4pre
+ *
+ *  Revision 1.32  1999/03/14 08:47:54  src
+ *  Uprated log for innreport.
+ *
+ *  Revision 1.31  1999/03/11 07:30:00  src
+ *  Implemented check for spool free space
+ *
+ *  Revision 1.30  1999/03/07 15:40:58  src
+ *  Removed DBZ_VERSION from configuration
+ *
+ *  Revision 1.29  1999/03/07 14:58:18  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.28  1999/03/05 11:20:43  src
+ *  Delayed decision of dbz v3 vz. 6 to runtime.
+ *
+ *  Revision 1.27  1998/11/23 12:25:49  src
+ *  Default window size 10.
+ *
+ *  Revision 1.26  1998/11/22 18:37:10  src
+ *  Filter test.
+ *
+ *  Revision 1.25  1998/11/22 18:02:32  src
+ *  Filtering repaired
+ *
+ *  Revision 1.24  1998/11/22 08:23:07  src
+ *  Added --forget-inactive
+ *
+ *  Revision 1.23  1998/11/22 08:00:08  src
+ *  Option --no-queue
+ *
+ *  Revision 1.22  1998/11/21 19:14:23  src
+ *  Added --filter option
+ *
+ *  Revision 1.21  1998/10/25 04:08:56  src
+ *  A little more debugging on lockfiles.
+ *
+ *  Revision 1.20  1998/10/23 06:10:57  src
+ *  Fixed --without-history
+ *
+ *  Revision 1.19  1998/10/21 07:31:22  src
+ *  Opened syslog at an an earlier stage
+ *
  *  Revision 1.18  1998/09/21 10:04:29  src
  *  Added new command line options for --inews
  *
@@ -55,8 +139,13 @@
 #include "common.h"
 #include "proto.h"
 #include "options.h"
+#include "self.h"
+#include "news.h"
 #include "patchlevel.h"
 
+#define INIT_CONFIG 1
+#include "newsconfig.h"
+
 #if HAVE_GETOPT_LONG
 #include <getopt.h>
 #else
@@ -67,9 +156,11 @@
 #  include <sys/stat.h>
 #endif
 
-static void 
-pickargs(int argc,char *argv[],char *envp[]);
-static void 
+static void
+load_profile(char *name, char *envp[],int complain);
+static void
+lnumarg(char *arg,long *where, char option);
+static void
 numarg(char *arg,int *where, char option);
 static void 
 usage(void);
@@ -79,6 +170,114 @@ static void
 timearg(char *arg,long *where, char dflt, char option);
 
 char *newline = "\r\n";
+static int did_stat_attempts = 0;
+
+/*
+ *  long command line options
+ */
+static int option_index = -1;
+static char *long_list = {
+    "a:b:cde:f:gh:ikl:mnopq:rs:t:uvw:x:y:zA:B:H:I:LM:N:P:RT:W:X:Z?"
+} ;
+static struct option long_options[] = {
+   /* char *name, int has_arg, int *flag, int val */
+    {"auth",            1, 0, 'a'},
+    {"size",            1, 0, 'b'},
+    {"cnews",           0, 0, 'c'},
+    {"verbose",         0, 0, 'd'},
+    {"end",             1, 0, 'e'},
+    {"posted",          1, 0, 'f'},
+    {"nofetch",         0, 0, 'g'},
+    {"no-fetch",        0, 0, 'g'},
+    {"history",         1, 0, 'h'},
+    {"inn",             0, 0, 'i'},
+    {"keep-path",       0, 0, 'k'},
+    {"keeppath",        0, 0, 'k'},
+    {"log",             1, 0, 'l'},
+    {"nomsgid",         0, 0, 'm'},
+    {"no-msgid",        0, 0, 'm'},
+    {"noaction",        0, 0, 'n'},
+    {"dry-run",         0, 0, 'n'},
+    {"keepold",         0, 0, 'o'},
+    {"nopost",          0, 0, 'p'},
+    {"no-post",         0, 0, 'p'},
+    {"enquire",         1, 0, 'q'},
+    {"reader",          0, 0, 'r'},
+    {"spool",           1, 0, 's'},
+    {"timeout",         1, 0, 't'},
+    {"noforce",         0, 0, 'u'},
+    {"no-force",        0, 0, 'u'},
+    {"version",         0, 0, 'v'},
+    {"chat",            1, 0, 'w'},
+    {"exec",            1, 0, 'x'},
+    {"connect",         1, 0, 'y'},
+    {"sync",            0, 0, 'z'},
+    {"active",          1, 0, 'A'},
+    {"batch",           1, 0, 'B'},
+    {"home",            1, 0, 'H'},
+    {"incoming",        1, 0, 'I'},
+    {"list",            1, 0, 'L'},
+    {"missing",         1, 0, 'M'},
+    {"newsfeeds",       1, 0, 'N'},
+    {"bounce",          1, 0, 'O'},
+    {"profile",         1, 0, 'P'},
+    {"stat",            1, 0, 'Q'},
+    {"reset",           0, 0, 'R'},
+    {"sys",             1, 0, 'S'},
+    {"togo",            1, 0, 'T'},
+    {"rnews",           0, 0, 'U'},
+    {"window",          1, 0, 'W'},
+    {"maxnew",          1, 0, 'X'},
+    {"syncnew",         0, 0, 'Z'},
+    {"help",            0, 0, '?'},
+    {"attach",          1, 0,  1},
+    {"desc",            1, 0,  2},
+    {"alldesc",         1, 0,  3},
+    {"ctlinnd",         1, 0,  4},
+    {"locks",           1, 0,  5},
+    {"mfilter",         1, 0,  6},
+    {"maxart",          1, 0,  7},
+    {"newline",         0, 0,  8},
+    {"nops",            0, 0,  9},
+    {"authgeneric",     0, 0, 10},
+    {"debug",           1, 0, 11},
+    {"fail",            1, 0, 12},
+    {"max-path",        1, 0, 13},
+    {"inews",           0, 0, 14},
+    {"inews-options",   1, 0, 15},
+    {"add-header",      1, 0, 16},
+    {"pipe-to",         1, 0, 17},
+    {"rnews-to",        1, 0, 18},
+    {"filter",          1, 0, 19},
+    {"groups",          1, 0, 20},
+    {"config",          0, 0, 21},
+    {"config-is",       1, 0, 22},
+    {"minfree",         1, 0, 23},
+    {"newlist",         1, 0, 24},
+    {"ihave",           0, 0, 25},
+    {"inhosts",         1, 0, 26},
+    {"spoolname",       1, 0, 27},
+    {"hostname",        1, 0, 28},
+    {"port",            1, 0, 29},
+    {"readbeforeauth",  0, &readbeforeauth_opt, 1}, /* do AUTHINFO after MODE READER */
+    {"no-path",         0, &nopath_opt,         1},
+    {"no-ps",           0, &no_ps,              1},
+    {"nohostlock",      0, &no_host_lock,       1},
+    {"no-hostlock",     0, &no_host_lock,       1},
+    {"nonext",          0, &no_next,            1},
+    {"no-next",         0, &no_next,            1},
+    {"continue",        0, &conterr,            1}, /* continue, ignoring some errors */
+    {"noreader",        0, &mode_reader_opt,   -1}, /* don't do a 'MODE READER' */
+    {"no-reader",       0, &mode_reader_opt,   -1},
+    {"noqueue",         0, &noqueue,            1},
+    {"no-queue",        0, &noqueue,            1},
+    {"forget-inactive", 0, &forget_inactive,    1},
+    {"keep-fake",       0, &keep_fake,          1},
+    {"scanlogs",        0, &scanlogs_lock,      1},
+    {"tag",             0, &tag_opt,            1},
+    {"verify",          0, &verify_opt,         1},
+    {0, 0, 0, 0}
+};
 
 /*
  *  main 
@@ -94,25 +293,72 @@ main(int argc, char *argv[], char *envp[])
     stringarg(q,&pname,0);
     begintitle(argc,argv,envp);
 
-    /* plus some command line defaults */
+    /* find out who we are */
+    my_pid = getpid();
+    gethostname(my_hostname, sizeof(my_hostname)-1);
+
+    log_open();
+
+    /* some command line defaults */
     nopull_opt = 0;
     inn_opt = INN_MODE;
     minspool = MINSPOOL;
     ai_isgeneric = 0;
     readbeforeauth_opt = 0;
-    stat_attempts = -1;
+    window = 10;
     max_path = 1;
 
     /* use the default umask for news */
     umask(NEWSUMASK);
 
+    /*
+     *  check if runtime configuration is required
+     *  BUG: what if newshome specified on command line
+     */
+    if (!cfg_newshome[0]) {
+	if (debug_opt)
+	    fprintf(stderr,"doing runtime config from \"%s\"\n",NEWSCONFIG);
+	load_config(NEWSCONFIG);
+    }
+
+    /*
+     *  load the standard profile, if any
+     *  NOTE: debug flags are not active yet...
+     */
+    if (cfg_etc[0]) {
+	char name[PATH_MAX];
+	build_filename(name,cfg_etc,"/",NEWSX_CONF,NULL);
+	load_profile(name, envp, 0);
+    }
+
     /* parse the args */
     pickargs(argc,argv,envp);
 
     /* defaults */
+    if (!hostname || !hostname[0]) hostname = spoolname;
+    if (!hostport || !hostport[0]) hostport = NNTP_PORT;
     if (!spoolname) spoolname = getenv("NNTPSERVER");
     if (!spooldir) stringarg(SPOOL,&spooldir,0);
-    if (stat_attempts < 0) stat_attempts = (window==0) ? 0:2;
+    if (!did_stat_attempts) stat_attempts = (window==0) ? 0:2;
+
+    /* sanity check */
+    if (!enquire_opt && (!spoolname || !spoolname[0])) {
+	fprintf(stderr, "spool name missing\n");
+	usage();
+	unlock_exit(2);
+    }
+
+    if (spoolname) {
+	char name[PATH_MAX+PATH_MAX];
+
+	/* set the program title for "ps" */
+	sprintf(name,"%s(%s)",pname,spoolname);
+	settitle(name);
+
+	/* load per-host profile */
+	build_filename(name,cfg_inhosts,"/",spoolname,_CONF);
+	load_profile(name, envp, 0);
+    }
 
     /* do what we were told */
     ok = doit();
@@ -126,105 +372,18 @@ main(int argc, char *argv[], char *envp[])
 
 /*
  *  pick arguments from command line
- *  NOTE: the actual command line will be cleared after this
- *        so all strings will be lost and must be thus be copied as reqd.
+ *  NOTE: the actual command line will be cleared after this,
+ *        so all strings will be lost and must thus be copied as reqd.
  */
-static void 
+void
 pickargs(int argc,char *argv[],char *envp[])
 {
     int c;
     char *p;
-    char name[PATH_MAX+PATH_MAX];
 
     for (;;) {
-	int option_index = 0;
-	static struct option long_options[] = {
-	   /* char *name, int has_arg, int *flag, int val */
-	    {"auth",            1, 0, 'a'},
-	    {"size",            1, 0, 'b'},
-	    {"cnews",           0, 0, 'c'},
-	    {"verbose",         0, 0, 'd'},
-	    {"end",             1, 0, 'e'},
-	    {"posted",          1, 0, 'f'},
-	    {"nofetch",         0, 0, 'g'},
-	    {"no-fetch",        0, 0, 'g'},
-	    {"history",         1, 0, 'h'},
-	    {"inn",             0, 0, 'i'},
-	    {"keep-path",       0, 0, 'k'},
-	    {"keeppath",        0, 0, 'k'},
-	    {"log",             1, 0, 'l'},
-	    {"nomsgid",         0, 0, 'm'},
-	    {"no-msgid",        0, 0, 'm'},
-	    {"noaction",        0, 0, 'n'},
-	    {"dry-run",         0, 0, 'n'},
-	    {"keepold",         0, 0, 'o'},
-	    {"nopost",          0, 0, 'p'},
-	    {"no-post",         0, 0, 'p'},
-	    {"enquire",         1, 0, 'q'},
-	    {"reader",          0, 0, 'r'},
-	    {"spool",           1, 0, 's'},
-	    {"timeout",         1, 0, 't'},
-	    {"noforce",         0, 0, 'u'},
-	    {"no-force",        0, 0, 'u'},
-	    {"version",         0, 0, 'v'},
-	    {"chat",            1, 0, 'w'},
-	    {"exec",            1, 0, 'x'},
-	    {"connect",         1, 0, 'y'},
-	    {"sync",            0, 0, 'z'},
-	    {"active",          1, 0, 'A'},
-	    {"batch",           1, 0, 'B'},
-	    {"ctlinnd",         1, 0, 'C'},
-	    {"debug",           1, 0, 'D'},
-	    {"continue",        0, 0, 'E'},
-	    {"fail",            1, 0, 'F'},
-	    {"newlist",         1, 0, 'G'},
-	    {"home",            1, 0, 'H'},
-	    {"incoming",        1, 0, 'I'},
-	    {"ihave",           0, 0, 'J'},
-	    {"inhosts",         1, 0, 'K'},
-	    {"list",            1, 0, 'L'},
-	    {"missing",         1, 0, 'M'},
-	    {"newsfeeds",       1, 0, 'N'},
-	    {"bounce",          1, 0, 'O'},
-	    {"locks",           1, 0, 'P'},
-	    {"stat",            1, 0, 'Q'},
-	    {"reset",           0, 0, 'R'},
-	    {"sys",             1, 0, 'S'},
-	    {"togo",            1, 0, 'T'},
-	    {"rnews",           0, 0, 'U'},
-	    {"window",          1, 0, 'W'},
-	    {"maxnew",          1, 0, 'X'},
-	    {"noreader",        0, 0, 'Y'},
-	    {"no-reader",       0, 0, 'Y'},
-	    {"syncnew",         0, 0, 'Z'},
-	    {"help",            0, 0, '?'},
-	    {"attach",          1, 0,  1},
-	    {"desc",            1, 0,  2},
-	    {"alldesc",         1, 0,  3},
-	    {"nohostlock",      0, 0,  4},
-	    {"no-hostlock",     0, 0,  4},
-	    {"nonext",          0, 0,  5},
-	    {"no-next",         0, 0,  5},
-	    {"mfilter",         1, 0,  6},
-	    {"maxart",          1, 0,  7},
-	    {"newline",         0, 0,  8},
-	    {"nops",            0, 0,  9},
-	    {"no-ps",           0, 0,  9},
-	    {"authgeneric",     0, 0, 10},
-	    {"readbeforeauth",  0, 0, 11},
-	    {"no-path",         0, 0, 12},
-	    {"max-path",        1, 0, 13},
-	    {"inews",           0, 0, 14},
-	    {"inews-options",   1, 0, 15},
-	    {"add-header",      1, 0, 16},
-	    {"pipe-to",         1, 0, 17},
-	    {"rnews-to",        1, 0, 18},
-	    {0, 0, 0, 0}
-	};
-
-	c = getopt_long(argc,argv,
-		"a:b:cde:f:gh:ikl:mnopq:rs:t:uvw:x:y:zA:B:H:I:LM:N:RT:W:X:Z?",
-		long_options, &option_index);
+	option_index = -1;
+	c = getopt_long(argc,argv, long_list, long_options, &option_index);
 	if (c == EOF) break;
 
 	switch (c) {
@@ -233,7 +392,7 @@ pickargs(int argc,char *argv[],char *envp[])
 	    get_authinfo(optarg);
 	    break;
 	case 'b':                       /* min spool size in bytes */
-	    numarg(optarg,&minspool,c);
+	    lnumarg(optarg,&minspool,c);
 	    break;
 	case 'c':                       /* C News mode */
 	    inn_opt=0;
@@ -242,18 +401,24 @@ pickargs(int argc,char *argv[],char *envp[])
 	    debug_opt++;
 	    break;
 	case 'e':                       /* end at tag */
-	    for (p=optarg; isspace(*p); ++p) ; /* remove any leading blanks */
+	    /* remove any leading blanks */
+	    for (p=optarg; isspace((unsigned char)(*p)); ++p) ;
 	    stringarg(p,&end_tag,c);
 	    break;
 	case 'f':                       /* folder */
 	    stringarg(optarg,&folder,c);
 	    break;
 	case 'h':                       /* history file */
-	    stringarg(optarg,&history,c);
-	    if (!history || !history[0]) {
+	    stringarg(optarg,&cfg_history,c);
+	    if (!cfg_history || !cfg_history[0]) {
 		/* disable history */
+#if WITH_HISTORY
 		extern int hist_open;
 		hist_open = -1;
+#else
+	    } else {
+		fprintf(stderr, "%s: history not supported\n", pname);
+#endif
 	    }
 	    break;
 	case 'g':                       /* don't get news */
@@ -284,9 +449,6 @@ pickargs(int argc,char *argv[],char *envp[])
 	case 'r':                       /* do a 'MODE READER' */
 	    mode_reader_opt = 1;
 	    break;
-	case 'Y':                       /* don't do a 'MODE READER' */
-	    mode_reader_opt = -1;
-	    break;
 	case 'q':                       /* enquire */
 	    stringarg(optarg,&enquire_opt,c);
 	    break;
@@ -322,59 +484,33 @@ pickargs(int argc,char *argv[],char *envp[])
 	    zap_opt++;
 	    break;
 	case 'A':                       /* active file */
-	    stringarg(optarg,&active,c);
+	    stringarg(optarg,&cfg_active,c);
 	    break;
 	case 'B':                       /* batch directory */
-	    stringarg(optarg,&batch,c);
-	    break;
-	case 'C':                       /* ctlinnd program */
-	    stringarg(optarg,&ctlpgm,c);
-	    break;
-	case 'D':                       /* debug */
-	    numarg(optarg,&debug_opt,c);
-	    break;
-	case 'E':                       /* continue, ignoring some errors */
-	    ++conterr;
-	    break;
-	case 'F':                       /* fail after specified time */
-	    if (strcmp(optarg,"never")==0) {
-		failtime=0;
-	    } else {
-		timearg(optarg,&failtime,'h',c);
-		if (failtime==0) failtime=1;
-	    }
-	    break;
-	case 'G':                       /* get list of new groups */
-	    stringarg(optarg,&group_newlist,c);
+	    stringarg(optarg,&cfg_batch,c);
 	    break;
 	case 'H':                       /* news home directory */
-	    stringarg(optarg,&newshome,c);
+	    stringarg(optarg,&cfg_newshome,c);
 	    break;
 	case 'I':                       /* incoming directory */
-	    stringarg(optarg,&incoming,c);
-	    break;
-	case 'J':                       /* ihave */
-	    if (nomsgid_opt) goto usage;
-	    ihave_opt++;
+	    stringarg(optarg,&cfg_incoming,c);
 	    break;
 	case 'L':                       /* get list of groups */
 	    stringarg(optarg,&group_list,c);
 	    break;
-	case 'K':                       /* in.hosts directory */
-	    stringarg(optarg,&inhosts,c);
-	    break;
 	case 'M':                       /* missing num */
 	    numarg(optarg,&stat_attempts,c);
+	    ++did_stat_attempts;
 	    break;
 	case 'N':                       /* newsfeeds file */
 	case 'S':                       /* sys file */
-	    stringarg(optarg,&newsfeeds,c);
+	    stringarg(optarg,&cfg_newsfeeds,c);
 	    break;
 	case 'O':                       /* bounce */
 	    stringarg(optarg,&bounce,c);
 	    break;
-	case 'P':                       /* locks */
-	    stringarg(optarg,&locks,c);
+	case 'P':                       /* profile */
+	    load_profile(optarg,envp,1);
 	    break;
 	case 'Q':                       /* stat */
 	    stringarg(optarg,&statfile,c);
@@ -431,11 +567,11 @@ pickargs(int argc,char *argv[],char *envp[])
 	case 3:                         /* get description of all groups */
 	    stringarg(optarg,&group_alldesc,c);
 	    break;
-	case 4:                         /* no host lock */
-	    ++no_host_lock;
+	case 4:                         /* ctlinnd program */
+	    stringarg(optarg,&ctlpgm,c);
 	    break;
-	case 5:                         /* no NEXT command */
-	    ++no_next;
+	case 5:                         /* locks */
+	    stringarg(optarg,&cfg_locks,c);
 	    break;
 	case 6:                         /* mfilter */
 	    stringarg(optarg,&mfilter,c);
@@ -446,9 +582,6 @@ pickargs(int argc,char *argv[],char *envp[])
 	case 8:                         /* newline */
 	    newline = "\n";
 	    break;
-	case 9:                         /* nops */
-	    ++no_ps;
-	    break;
 	case 10:
 	    /* --authgeneric : authenticate using AUTHINFO GENERIC */
 	    /* if we've already parsed an argument specifying authentication */
@@ -472,11 +605,16 @@ pickargs(int argc,char *argv[],char *envp[])
 	     */
 	    ai_isgeneric = 1;
 	    break;
-	case 11:                        /* do AUTHINFO after MODE READER */
-	    readbeforeauth_opt++;
+	case 11:                        /* debug */
+	    numarg(optarg,&debug_opt,c);
 	    break;
-	case 12:                        /* don't look at paths */
-	    nopath_opt++;
+	case 12:                        /* fail after specified time */
+	    if (strcmp(optarg,"never")==0) {
+		failtime=0;
+	    } else {
+		timearg(optarg,&failtime,'h',c);
+		if (failtime==0) failtime=1;
+	    }
 	    break;
 	case 13:                        /* maxpath */
 	    numarg(optarg,&max_path,c);
@@ -513,10 +651,45 @@ pickargs(int argc,char *argv[],char *envp[])
 	    stringarg(optarg,&rnews_path,c);
 	    rnews_name = rnews_path;
 	    break;
+	case 19:                        /* filter */
+	    stringarg(optarg,&filter,c);
+	    break;
+	case 20:                        /* groups */
+	    add_extra(optarg);
+	    break;
+	case 21:                        /* load standard config */
+	    load_config(NEWSCONFIG);
+	    break;
+	case 22:                        /* config file, as specified */
+	    load_config(optarg);
+	    break;
+	case 23:                        /* min spool free in kbytes */
+	    lnumarg(optarg,&minfree,c);
+	    break;
+	case 24:                        /* get list of new groups */
+	    stringarg(optarg,&group_newlist,c);
+	    break;
+	case 25:                        /* ihave */
+	    if (nomsgid_opt) goto usage;
+	    ihave_opt++;
+	    break;
+	case 26:                        /* in.hosts directory */
+	    stringarg(optarg,&cfg_inhosts,c);
+	    break;
+	case 27:                        /* spoolname */
+	    stringarg(optarg,&spoolname,c);
+	    break;
+	case 28:                        /* hostname */
+	    stringarg(optarg,&hostname,c);
+	    break;
+	case 29:                        /* port */
+	    stringarg(optarg,&hostport,c);
+	    break;
+	case 0:
+	    break;
 		
 	default:
 	  usage:
-	    /* BUG: have a more complete list! */
 	    usage();
 	    unlock_exit(2);
 	}
@@ -524,48 +697,31 @@ pickargs(int argc,char *argv[],char *envp[])
 	
     /* get spool name */
     if (optind < argc) {
-	if (argv[optind][0]) {
-	    stringarg(argv[optind],&spoolname,0);
-	}
+	if (argv[optind][0]) stringarg(argv[optind],&spoolname,0);
 	++optind;
     } 
-    if (!enquire_opt && (!spoolname || !spoolname[0])) {
-	fprintf(stderr, "spool name missing\n");
-	goto usage;
-    }
 
     /* get server name */
     if (optind < argc) {
-	if (argv[optind][0]) {
-	    stringarg(argv[optind],&hostname,0);
-	}
+	if (argv[optind][0]) stringarg(argv[optind],&hostname,0);
 	++optind;
     }
-    if (!hostname || !hostname[0]) hostname = spoolname;
 	
     /* get port name */
     if (optind < argc) {
-	if (argv[optind][0]) {
-	    stringarg(argv[optind],&hostport,0);
-	}
+	if (argv[optind][0]) stringarg(argv[optind],&hostport,0);
 	++optind;
     } 
-    if (!hostport || !hostport[0]) hostport = NNTP_PORT;
 
     if (optind < argc) {
 	fprintf(stderr, "excess arguments\n");
 	goto usage;
     }
-
-    /* set the program title for "ps" */
-    if (spoolname) {
-	sprintf(name,"%s(%s)",pname,spoolname);
-	settitle(name);
-    }
 }
 
 /*
  *  say how to use it
+ *  BUG: have a more complete list!
  */
 static void 
 usage(void)
@@ -586,31 +742,73 @@ usage(void)
     printf("\tman 8 newsx\n");
 }
 
+/*
+ *  load a profile
+ */
+static void
+load_profile(char *name, char *envp[], int complain)
+{
+    static int include = 0;
+    if (++include >= 20) {
+	fprintf(stderr,"too deep --profile nesting for \"%s\"\n",
+							    name);
+	unlock_exit(2);
+    }
+    split_file(name,envp,complain);
+    --include;
+}
+
+/*
+ *  make proper name of option, for error reporting
+ *  returns static buffer, so beware
+ */
+static char *
+namopt(char short_option)
+{
+    static char buf[50];
+
+    if (option_index >= 0) { /* it was a long option */
+	sprintf(buf,"--%s",long_options[option_index].name);
+    } else {
+	sprintf(buf,"-%c",short_option);
+    }
+    return buf;
+}
+
+/*
+ *  handle numeric long argument
+ */
+static void
+lnumarg(char *arg,long *where, char option)
+{
+    long a;
+    char *p;
+
+    a = strtol(arg,&p,10);
+    if (p == arg || *p) {
+	fprintf(stderr,"%s: bad numeric spec for option %s: %s\n",
+			pname,              namopt(option), arg);
+	return;
+    }
+
+#if 0 /* BUG: find other way of doing this */
+    if (*where != 0 && *where != a) {
+	fprintf(stderr,"%s: option %s was \"%d\", respecified as \"%d\"\n",
+		     pname, namopt(option), *where,                a);
+    }
+#endif
+    *where = a;
+}
+
 /*
  *  handle numeric argument
  */
 static void 
 numarg(char *arg,int *where, char option)
 {
-    int a;
-    char *p;
+    long a = *where;
 
-    a = strtol(arg,&p,10);
-    if (p == arg || *p) {
-	fprintf(stderr,"%s: bad numeric spec for option -%c: %s\n",
-			pname,                       option,  arg);
-	return;
-    }
-
-    if (*where && *where != a) {
-	if (option) {
-	    fprintf(stderr,"%s: option -%c was \"%d\", respecified as \"%d\"\n",
-			    pname,      option, *where,              a);
-	} else {
-	    fprintf(stderr,"%s: argument was \"%d\", respecified as \"%d\"\n",
-			    pname,            *where,              a);
-	}
-    }
+    lnumarg(arg,&a,option);
     *where = a;
 }
 
@@ -622,13 +820,8 @@ static void
 stringarg(char *arg,char **where, char option)
 {
     if (*where) {
-	if (option) {
-	    fprintf(stderr,"%s: option -%c was \"%s\", respecified as \"%s\"\n",
-			    pname,      option, *where,              arg);
-	} else {
-	    fprintf(stderr,"%s: argument was \"%s\", respecified as \"%s\"\n",
-			    pname,            *where,              arg);
-	}
+	fprintf(stderr,"%s: option %s was \"%s\", respecified as \"%s\"\n",
+		     pname, namopt(option), *where,                arg);
     }
     *where = xstrcpy(arg);
 }
@@ -665,16 +858,16 @@ timearg(char *arg,long *where, char dflt, char option)
     char *p;
 
     if (*where) {
-	fprintf(stderr,"%s: option -%c was respecified as \"%s\"\n",
-			pname,      option,                arg);
+	fprintf(stderr,"%s: option %s was respecified as \"%s\"\n",
+			pname, namopt(option),             arg);
     }
     *where = 0;
     do {
 	++a;
 	n = strtol(arg,&p,10);
 	if (p == arg) {
-	    fprintf(stderr,"%s: bad time spec for option -%c: %s\n",
-			    pname,                   option,  arg);
+	    fprintf(stderr,"%s: bad time spec for option %s: %s\n",
+			    pname,            namopt(option), arg);
 	    break;
 	}
 	if (timemul(*p,&n)) ++p;
