@@ -1,4 +1,4 @@
-/*  VER 021  TAB P   $Id: msgid.c,v 1.5 1998/09/09 07:32:12 src Exp $
+/*  VER 022  TAB P   $Id: msgid.c,v 1.6 1999/03/07 14:58:18 src Exp $
  *
  *  handle the message IDs
  *  to ensure we don't read a message twice
@@ -7,6 +7,9 @@
  *  the GNU General Public License applies
  *
  *  $Log: msgid.c,v $
+ *  Revision 1.6  1999/03/07 14:58:18  src
+ *  Read newsconfig supported. Storage API supported.
+ *
  *  Revision 1.5  1998/09/09 07:32:12  src
  *  Version 1.1
  *
@@ -50,10 +53,7 @@ new_msgid(char *msgid)
 
     /* add to list */
     n = sizeof(MESSAGE_ID) + strlen(msgid);
-    if (!(mp = malloc(n))) {
-	/* out of memory */
-	return 1;
-    }
+    mp = xmalloc(n);
     strcpy(mp->name,msgid);
     mp->next = messageids[h];
     messageids[h] = mp;
