@@ -1,4 +1,4 @@
-/*  VER 136  TAB P   $Id: proto.h,v 1.17 1998/09/21 10:04:29 src Exp $
+/*  VER 166  TAB P   $Id: proto.h,v 1.31 1999/04/19 03:32:39 src Exp $
  *
  *  function prototypes
  *
@@ -6,6 +6,48 @@
  *  the GNU General Public License applies
  *
  *  $Log: proto.h,v $
+ *  Revision 1.31  1999/04/19 03:32:39  src
+ *  Standard profile in $PATHETC/newsx.conf
+ *
+ *  Revision 1.30  1999/04/07 20:09:16  src
+ *  Implemented configuration reading, and eval mechanism
+ *
+ *  Revision 1.29  1999/04/07 08:02:11  src
+ *  Implemented --profile
+ *
+ *  Revision 1.28  1999/04/05 19:01:51  src
+ *  Lock for scanlogs
+ *
+ *  Revision 1.27  1999/03/24 03:53:00  src
+ *  Implemented "newsx" as magic exclude pattern
+ *
+ *  Revision 1.26  1999/03/16 08:12:46  src
+ *  Moved clean_active() to active.c
+ *
+ *  Revision 1.25  1999/03/16 08:03:30  src
+ *  Renamed lock() to do_lock()
+ *
+ *  Revision 1.24  1999/03/11 07:30:01  src
+ *  Implemented check for spool free space
+ *
+ *  Revision 1.23  1999/03/07 14:58:19  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.22  1998/11/22 18:02:32  src
+ *  Filtering repaired
+ *
+ *  Revision 1.21  1998/11/22 08:23:07  src
+ *  Added --forget-inactive
+ *
+ *  Revision 1.20  1998/11/22 08:00:08  src
+ *  Option --no-queue
+ *
+ *  Revision 1.19  1998/11/21 19:14:24  src
+ *  Added --filter option
+ *
+ *  Revision 1.18  1998/10/25 04:08:56  src
+ *  A little more debugging on lockfiles.
+ *
  *  Revision 1.17  1998/09/21 10:04:29  src
  *  Added new command line options for --inews
  *
@@ -36,6 +78,10 @@
  *  Support @-syntax in newsfeeds, and support AUTHINFO GENERIC
  */
 
+/* main.c */
+void
+pickargs(int argc,char *argv[],char *envp[]);
+
 /* doit.c */
 int 
 doit(void);
@@ -57,36 +103,36 @@ char *
 submit_article(char *articlename);
 int
 is_tag(char *line, char *tag);
+void
+close_article(void);
 
 /* lock.c */
 int
-lock(char *dir_name,char *lock_name);
+do_lock(char *dir_name,char *lock_name,int giveup);
 void 
 unlock_one(int id);
 void 
 unlock_all(void);
 void 
 unlock_exit(int n);
+int
+check_lock(char* name);
 
 /* util.c */
-char *
-news_home(void);
 int 
 append_file(FILE *f,char *name,FILE *af);
 int 
 rename_file(char *from,char *to);
-time_t
-what_date(FILE *f,char *name);
-time_t
-how_old(FILE *f, char *name);
 int 
-is_regular(FILE *f, char *name);
+is_regular(FILE *f, char *name, long *sizep);
 void 
 build_alt_filename(char *where,char *home,char *fixed,char *alt);
 void 
 build_filename(char *where,char *arg1,char *arg2,char *arg3,char *arg4);
 char *
 text_time(time_t t);
+char *
+skipsp(char *p);
 void *
 xmalloc(unsigned int size);
 void * 
@@ -101,6 +147,8 @@ void
 show_activity(void);
 void 
 log_msg(int type, char *fmt, ...);
+char *
+str_error(int err);
 
 /* server.c */
 SOCKET_D *current_socket();
@@ -111,6 +159,8 @@ close_server(int urgent);
 int 
 get_server_nntp(char *line, int size);
 int 
+get_server_nntp_nf(char *line, int size);
+int 
 get_server_msg(char *line, int size);
 int
 put_server_nolog(char *line);
@@ -135,11 +185,12 @@ put_socket(char *line, SOCKET_D *sock);
 int 
 select_group(char *group,long *firstp,long *lastp);
 int 
-fetch_article(long where,int more);
+fetch_article(long where,int more,char* group);
 
 /* request.c */
 int 
-put_request(char *request, int (*fn)(), long where);
+put_request(char *request, int (*fn)(char *,long,char *), 
+					long where, char *group);
 int 
 process_request(void);
 int 
@@ -165,13 +216,13 @@ is_in_active(char *group);
 int 
 unseen_active(char *group);
 void 
-load_active(void);
-void
-clear_active(void);
+load_active(int again);
 int
 bad_syntax(char *buf,int was,int expect,char *where);
 
 /* sys.c */
+void
+add_extra(char *group_list);
 void 
 load_sys(void);
 int 
@@ -211,6 +262,8 @@ program_open(char *program, SOCKET_D *sock);
 FILE *
 fork_write(char *path, int *pidp, char *arg1, char *arg2);
 FILE *
+fork_read(char *path,int *pidp,char *arg);
+FILE *
 open_any(char *any,int *pidp);
 int     
 close_any(FILE *f,int pid);
@@ -241,7 +294,8 @@ wildmat(char *text, char *p);
 
 /* bounce.c */
 int  
-bounce_msg(char *to, FILE *art,char *filename, char *msgid, char *reason);
+bounce_msg(char *to, char *a_data,int a_len,time_t a_arrived,
+			char *filename, char *msgid, char *reason);
 
 /* rnews.c */
 FILE *
@@ -249,6 +303,14 @@ open_rnews(int *pid);
 int 
 close_rnews(FILE *f, int pid);
 
+/* filter.c */
+int  
+do_filter(char *msgid, char *group);
+void
+filter_line(char *line);
+int  
+filter_reject(char *msgid);
+
 /* mfilter.c */
 int  
 do_mfilter(char *msgid);
@@ -265,8 +327,20 @@ add_exclusion(char *path);
 char *
 get_exclusion(void);
 int
+path_match(char *path);
+int
 traverse_path(char *path);
 int
 path_bangs(char *path);
 
+/* config.c */
+int
+load_config(char *configname);
 
+/* freespc.c */
+int 
+freespace(char *path, long *kbytep, long *inodep);
+
+/* split.c */
+void
+split_file(char *name,char *envp[],int complain);
