@@ -1,4 +1,4 @@
-/*  VER 139  TAB P   $Id: pull.c,v 1.12 1998/09/09 07:32:13 src Exp $
+/*  VER 146  TAB P   $Id: pull.c,v 1.17 1999/03/15 07:45:13 src Exp $
  *
  *  pull news      
  *
@@ -6,6 +6,21 @@
  *  the GNU General Public License applies
  *
  *  $Log: pull.c,v $
+ *  Revision 1.17  1999/03/15 07:45:13  src
+ *  Fix for unused spool
+ *
+ *  Revision 1.16  1999/03/07 14:58:19  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.15  1999/03/04 17:19:27  src
+ *  Fix for removed groups that weren't
+ *
+ *  Revision 1.14  1998/11/22 08:23:07  src
+ *  Added --forget-inactive
+ *
+ *  Revision 1.13  1998/11/21 19:14:24  src
+ *  Added --filter option
+ *
  *  Revision 1.12  1998/09/09 07:32:13  src
  *  Version 1.1
  *
@@ -23,6 +38,7 @@
 #include "proto.h"
 #include "options.h"
 #include "statistics.h"
+#include "newsconfig.h"
 #include "news.h"
 
 /* BUG: have NNTP timeout function with unlimited line length */
@@ -39,6 +55,7 @@ extern long latest_where;   /* external: last article read OK */
  */
 static int pull_active;
 static int pull_anything;
+static int removed_groups;             /* number of groups removed */
 static FILE *pull_in;
 static FILE *pull_tmp;
 static char activename[PATH_MAX];
@@ -72,8 +89,8 @@ pull_group(char *group, long *wherep, int is_new)
     default:
 	return 0; /* no point in continuing */
     }
-    log_msg(L_DEBUGMORE,"group selected %s, %ld-%ld",group,first,last);
     /* BUG: check for consistency?! */
+    log_msg(L_DEBUGMORE,"group selected %s, %ld-%ld",group,first,last);
 
     if (zap_opt || (is_new && zapnew_opt)) {
 	/* silently update to latest article */
@@ -92,39 +109,42 @@ pull_group(char *group, long *wherep, int is_new)
 
     /*
      *  fetch all articles
+     *  where==0 and last==0 is special case for unused spool
      */
-    if (is_new && max_new && where < last+1-max_new) {
-	log_msg(L_DEBUG,"skipping %ld articles due to --maxnew",
-						(last+1-where) - max_new);
-	where = last+1-max_new;
-    }
-    if (max_articles && where < last+1-max_articles) {
-	log_msg(L_DEBUG,"skipping %ld articles due to --maxart",
-					    (last+1-where) - max_articles);
-	where = last+1-max_articles;
-    }
-
-    latest_where = -1L;
-    while (where <= last) {
-	if (!fetch_article(where,first_time)) {
-	     more = 0;
-	     break;
+    if (where > 0 || last > 0) {
+	if (is_new && max_new && where < last+1-max_new) {
+	    log_msg(L_DEBUG,"skipping %ld articles due to --maxnew",
+						    (last+1-where) - max_new);
+	    where = last+1-max_new;
 	}
-	first_time = 0;
-	/* BUG: NOTE that 400 is used for aborting a transfer... */
-	++where;
+	if (max_articles && where < last+1-max_articles) {
+	    log_msg(L_DEBUG,"skipping %ld articles due to --maxart",
+						(last+1-where) - max_articles);
+	    where = last+1-max_articles;
+	}
 
-	if (bytes_in_spool > minspool) {
-	    /* flush spool if too large */
-	    /* BUG: L_DEBUGMORE */
-	    log_msg(L_DEBUG,"flushing spool...");
-	    flush_spool(0);
-	    bytes_in_spool = 0;
+	latest_where = -1L;
+	while (where <= last) {
+	    if (!fetch_article(where,first_time,group)) {
+		 more = 0;
+		 break;
+	    }
+	    first_time = 0;
+	    /* BUG: NOTE that 400 is used for aborting a transfer... */
+	    ++where;
+
+	    if (bytes_in_spool > minspool) {
+		/* flush spool if too large */
+		/* BUG: L_DEBUGMORE */
+		log_msg(L_DEBUG,"flushing spool...");
+		flush_spool(0);
+		bytes_in_spool = 0;
+	    }
+	}
+	/* flush all remaining requests */
+	if (!flush_request()) {
+	    more = 0;
 	}
-    }
-    /* flush all remaining requests */
-    if (!flush_request()) {
-	more = 0;
     }
 
     /* update article index */
@@ -170,10 +190,18 @@ pull_cleanup(int urgent)
     }
 
     /* clean up and update host active */
-    if (pull_anything) {
-	log_msg(l_debug,"%d group%s contained new articles",
+    if (pull_anything || removed_groups) {
+	if (pull_anything) {
+	    log_msg(l_debug,"%d group%s contained new articles",
 			pull_anything,
 			pull_anything==1 ? "":"s");
+	}
+	if (removed_groups) {
+	    log_msg(l_debug,"removed %d group%s",
+			removed_groups,
+			removed_groups==1 ? "":"s");
+	}
+	removed_groups =
 	pull_anything = 0;
 
 	if (active_old[0]) {
@@ -201,20 +229,19 @@ pull(char *spoolname)
     int n;
     long where,where0;
     char group[BUFSIZ];
-    char inhostsdir[PATH_MAX];
 
     progtitle("pull: opening host active");
 
-    pull_anything = 0;
+    removed_groups =
+    pull_anything =
     pull_active = 0;
 
     /* BUG: add procid or something... */
     /* BUG: make directory if not there ? */
 
-    build_alt_filename(inhostsdir,SPOOL,INHOSTS,inhosts);
-    build_filename(activename,inhostsdir,"/",spoolname,NULL);
-    build_filename(active_tmp,inhostsdir,"/",spoolname,_TMP);
-    build_filename(active_old,inhostsdir,"/",spoolname,_OLD);
+    build_filename(activename,cfg_inhosts,"/",spoolname,NULL);
+    build_filename(active_tmp,cfg_inhosts,"/",spoolname,_TMP);
+    build_filename(active_old,cfg_inhosts,"/",spoolname,_OLD);
 
     if (!(pull_in = fopen(activename,"r"))) {
 	log_msg(L_ERRno,"can't find a host active \"%s\"",activename);
@@ -253,6 +280,7 @@ pull(char *spoolname)
 		}
 		fprintf(pull_tmp,":%s\n",t);
 	    } else {
+		/* must be a group name */
 		where = 0;
 		if ((n=sscanf(p,"%[^ \n\t!:] %ld",group,&where)) < 1) {
 		    if (!skip) {
@@ -273,11 +301,20 @@ pull(char *spoolname)
 				fflush(pull_tmp);
 			    }
 			} else {
-			    log_msg(L_DEBUGMORE,"group %s not active",group);
+			    if (forget_inactive) {
+				/* loose all information about groups
+				   no longer in active file */
+				log_msg(L_INFO,"group %s no longer active - removed from host list",
+						group);
+				group[0] = '\0';
+				++removed_groups;
+			    } else {
+				log_msg(L_DEBUGMORE,"group %s not active",group);
+			    }
 			}
 		    }
 		    /* updated count */
-		    fprintf(pull_tmp,"%s %ld\n",group,where);
+		    if (group[0]) fprintf(pull_tmp,"%s %ld\n",group,where);
 		}
 	    }
 	}
