@@ -1,11 +1,47 @@
-/*  VER 162   TAB P   $Id: putarticle.c,v 1.11 1998/09/11 16:37:44 src Exp $
+/*  VER 193   TAB P   $Id: putarticle.c,v 1.23 1999/04/05 19:01:51 src Exp $
  *
  *  POST articles via an NNTP server
  *
- *  copyright 1996, 1997 Egil Kvaleberg, egil@kvaleberg.no
+ *  copyright 1996, 1997, 1999 Egil Kvaleberg, egil@kvaleberg.no
  *  the GNU General Public License applies
  *
  *  $Log: putarticle.c,v $
+ *  Revision 1.23  1999/04/05 19:01:51  src
+ *  Lock for scanlogs
+ *
+ *  Revision 1.22  1999/04/05 13:07:34  src
+ *  Various fixes for storeage-API
+ *
+ *  Revision 1.21  1999/03/31 03:28:20  src
+ *  Also removing "NNTP-Posting-Date" when posting
+ *
+ *  Revision 1.20  1999/03/19 13:01:50  src
+ *  Removed comma from log file format
+ *
+ *  Revision 1.19  1999/03/19 08:02:26  src
+ *  Fixed problem with newlines in logfile introduced in 1.4pre
+ *
+ *  Revision 1.18  1999/03/16 08:03:31  src
+ *  Renamed lock() to do_lock()
+ *
+ *  Revision 1.17  1999/03/15 08:50:28  src
+ *  Fixed posting with new storeageAPI
+ *
+ *  Revision 1.16  1999/03/14 08:47:55  src
+ *  Uprated log for innreport.
+ *
+ *  Revision 1.15  1999/03/11 07:54:47  src
+ *  Moved storage to lib and src
+ *
+ *  Revision 1.14  1999/03/07 14:58:19  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.13  1999/03/04 17:43:20  src
+ *  Removing X-trace etc.
+ *
+ *  Revision 1.12  1998/11/22 08:00:08  src
+ *  Option --no-queue
+ *
  *  Revision 1.11  1998/09/11 16:37:44  src
  *  Lockfile for logfile of posted articles and for posted article folder.
  *
@@ -32,6 +68,8 @@
 #include "statistics.h"
 #include "news.h"
 #include "nntp.h"
+#include "self.h"
+#include "sim.h"
 
 /*
  *  local stuff
@@ -45,6 +83,8 @@ static char msgsender[NNTP_STRLEN];
 
 static int article_lines = 0;
 
+static int SM_open = 0;
+
 #ifndef HAVE_STRTOUL
 /*
  *  for systems that doesn't have it, assuming base<=10
@@ -126,12 +166,13 @@ check_header(char *line, int *skip)
 	}
 	break;
     case 'N':
-	if (is_tag(line,"NNTP-Posting-Host:") && !ihave_opt) {
+	if ((is_tag(line,"NNTP-Posting-Host:")
+	  || is_tag(line,"NNTP-Posting-Date:")) && !ihave_opt) {
 	    /*
-	     * when POSTing, this header may cause
+	     * when POSTing, these headers may cause
 	     * messages to be rejected
 	     * fix by: Riku Saikkonen <rjs@isil.lloke.dna.fi>               
-	     *                 and Simon J. Mudd <sjmudd@bitmailer.net>
+	     *         and Simon J. Mudd <sjmudd@bitmailer.net>
 	     */
 	    *skip = 1;
 	    return;
@@ -176,7 +217,9 @@ check_header(char *line, int *skip)
 	    /* always omit local Xref */
 	    *skip = 1;
 	    return;
-	} else if (is_tag(line,"X-Server-Date:")) {
+	} else if (is_tag(line,"X-Server-Date:")
+		|| is_tag(line,"X-Trace:")
+		|| is_tag(line,"X-Complaints-To:")) {
 	    if (!ihave_opt) {
 		/* should not occur in postings */
 		*skip = 1;
@@ -198,37 +241,36 @@ check_header(char *line, int *skip)
  *  sniff an article to find its message ID
  */
 static char *
-sniff_article(FILE *fp)
+sniff_article(ARTHANDLE *article)
 {
-    char line[NNTP_STRLEN+2+1];
+    char *line = article->data;
     int len;
     int dummy;
-    int multi_1st = 0;
-    int is_newline = 1;
+    char *p;
+    char c;
 
     msgid[0] = '\0';
 	
     /* read the article, header and body */
     progtitle("find message-ID");
     for (;;) {
-	if (!fgets(line, NNTP_STRLEN, fp)) break;
-	if ((len=strlen(line)) == 0) break;
+	/* BUG: article->len */
+	if (!(p = strchr(line,'\r')) || p[1]!='\n') {
+	    /* should not happen */
+	    break;
+	}
+	len = (p+2) - line;
 
-	multi_1st = is_newline; /* full line? */
-	if (line[len-1] == '\n') {
-	    is_newline = 1;
-	    line[len-1] = '\0';
-	} else {
-	    is_newline = 0;
-	}
-	if (multi_1st) {
-	    if (len == 0) break; /* empty line: end of header */
-	    check_header(line, &dummy);
-	    if (msgid[0]) break; /* found header */
-	}
+	if (len == 2) break; /* empty line: end of header */
+
+	c = *p;
+	*p = '\0';
+	check_header(line, &dummy);
+	*p = c;
+	if (msgid[0]) break; /* found header */
+
+	line += len;
     }
-    /* rewind file */
-    fseek(fp,0L,0);
     return msgid[0] ? msgid : 0;
 }
 
@@ -237,16 +279,16 @@ sniff_article(FILE *fp)
  *  set static variable article_lines
  */
 static void
-write_article(FILE *fp)
+write_article(ARTHANDLE *article)
 {
-    char line[NNTP_STRLEN+2+1];
+    char *line = article->data;
     int len;
     int hdr = 1;
     int skiphdr = 0;
     FILE *folder_file = 0;
     int lock_id = -1;
-    int multi_1st = 0;
-    int is_newline = 1;
+    char c0,c1,c2;
+    char *p;
 
     msgid[0] = '\0';
     msgsender[0] = '\0';
@@ -255,50 +297,40 @@ write_article(FILE *fp)
     /* transfer the article, header and body */
     progtitle("post: transfer article");
     for (;;) {
-	line[NNTP_STRLEN] = '\0'; 
-	if (!fgets(line, NNTP_STRLEN, fp)) break;
-
-	/* must remove trailing linefeed */
-	if ((len=strlen(line)) == 0) break;
-
-	multi_1st = is_newline; /* if previous line ended here */
-	if (line[len-1] == '\n') {
-	    /* a complete line, or line termination */
-	    is_newline = 1;
-	    line[--len] = '\0';
-	} else {
-	    /*
-	     * no newline detected - handle very long lines too
-	     * problem pinpointed by Riku Saikkonen <rjs@isil.lloke.dna.fi>
-	     */
-	    is_newline = 0;
+	/* BUG: API what about length? */
+	if (!(p = strchr(line,'\r')) || p[1]!='\n') {
+	    /* panic */
+	    log_msg(L_ERR,"inconsistent article format");
+	    exit_cleanup(1);
 	}
-	if (multi_1st) {
-	    if (len == 0) {
-		/* empty line: end of header */
-		hdr = skiphdr = 0;
-	    } else if (len == 1 && line[0] == '.') {
-		/*
-		 * posting contains EOF, so convert to something
-		 * harmless according to RFC-977, section 3.10.1
-		 * fix by Riku Saikkonen <rjs@isil.lloke.dna.fi>
-		 */
-		strcpy(line,"..");
-		len = 2;
-	    } 
-	    if (hdr) {
-		/* update skip state */
-		check_header(line, &skiphdr);
-	    }
-	    ++article_lines;
+
+	len = (p+2) - line;
+	++article_lines;
+
+	if (len == 1+2 && line[0] == '.') {
+	    /* proper end-of-file */
+	    break;
 	}
+
+	if (len == 2) {
+	    /* empty line: end of header */
+	    hdr = skiphdr = 0;
+	} else if (hdr) {
+	    /* update skip state */
+	    /* BUG: API dirty? */
+	    c0 = *p;
+	    *p = '\0';
+	    check_header(line, &skiphdr);
+	    *p = c0;
+	}
+
 	if (!skiphdr) {
-	    if (is_newline) {
-		strcpy(line+len,newline);
-		len += strlen(newline);
-	    }
+	    /* BUG: API dirty? */
+	    c0 = *p; c1 = *(p+1); c2 = *(p+2);
+	    strcpy(p,newline);
 	    if (!put_server_msg(line)) exit_cleanup(9);
-	
+	    *p = c0; *(p+1) = c1; *(p+2) = c2;
+
 	    /* save to folder also */
 	    if (folder) {
 		if (!folder_file) {
@@ -309,7 +341,7 @@ write_article(FILE *fp)
 		     */
 		    progtitle("locking folder");
 		    build_filename(lockname,folder,LOCK_SUFFIX,NULL,NULL);
-		    lock_id = lock("",lockname);
+		    lock_id = do_lock("",lockname,0);
 
 		    if (!(folder_file = fopen(folder,"a"))) {
 			log_msg(L_ERRno,"can't open folder: %s",folder);
@@ -322,26 +354,22 @@ write_article(FILE *fp)
 		}
 		if (folder_file) {
 		    /* save to folder too */
-		    if (multi_1st 
-		     && strncmp(line, MAILFOLDER_TAG, MAILFOLDER_TAG_LEN)==0) {
+		    if (strncmp(line, MAILFOLDER_TAG, MAILFOLDER_TAG_LEN)==0) {
 			/* mail folder hack */
 			fputc('>', folder_file);
 		    }
-		    if (is_newline) {
-			/* back to Unix convention */
-			line[len-1] = '\0';
-			line[len-2] = '\n';
-		    }
+		    /* BUG: API dirty? */
+		    /* back to Unix convention */
+		    c0 = *p; c1 = *(p+1);
+		    *p = '\n';
+		    *(p+1) = '\0';
 		    fputs(line, folder_file);
+		    *p = c0; *(p+1) = c1;
 		}
 	    }
 	    net_bytecount += len;
 	}
-    }
-    if (!is_newline && !skiphdr) {
-	/* no trailing newline, so we need to add one */
-	if (!put_server_msg(newline)) exit_cleanup(9);
-	if (folder_file) fputc('\n', folder_file);
+	line += len;
     }
 
     if (folder_file) {
@@ -360,12 +388,6 @@ write_article(FILE *fp)
     if (lock_id >= 0)
 	unlock_one(lock_id);
 
-    if (ferror(fp)) {
-	/* panic */
-	log_msg(L_ERRno,"error transferring article");
-	exit_cleanup(1);
-    }
-
     /* send termination */
     progtitle("post: send termination");
     sprintf(line, ".%s", newline);
@@ -381,7 +403,7 @@ write_article(FILE *fp)
  *  return reason for failure
  */
 static char *
-put_article(FILE *fp, char **reason)
+put_article(ARTHANDLE *article, char **reason)
 {
     static char status[NNTP_STRLEN+1];
     char *p;
@@ -398,10 +420,11 @@ put_article(FILE *fp, char **reason)
     } else {
 	sprintf(status,"%d",CONT_POST); /* fake OK */
     }
+
     switch (strtoul(status,&endptr,10)) {
     case CONT_POST:                 /* post: posting allowed, continue */
     case CONT_XFER:                 /* ihave: go ahead */
-	write_article(fp);
+	write_article(article);
 	break;
 
     case ERR_POSTFAIL:              /* post: failed for some other reason */
@@ -552,20 +575,23 @@ log_to_file(char *articlename, char *ok)
      */
     progtitle("locking log file");
     build_filename(lockname,logfile,LOCK_SUFFIX,NULL,NULL);
-    lock_id = lock("",lockname);
+    lock_id = do_lock("",lockname,0);
 
     progtitle("post: logging");
     if (!(fp = fopen(logfile,"a"))) {
 	log_msg(L_ERRno,"can't open logfile: %s",logfile);
     } else {
-	fprintf(fp,"%s %s %s %s %s %s, %d lines\n",
+	/* for innreport; Uli Zappe <uli@tallowcross.uni-frankfurt.de> */
+	fprintf(fp,"%s %s newsx[%i]: %s\t%s\t%s\t%s\t%s\t%d lines\n",
 		    current_time() + 4, /* skip "Day " */
-		       spoolname,
-			  msgid[0] ? msgid : "<?>",
-			     articlename,
-				msgsender[0] ? msgsender : "?",
-				   ok,
-				       article_lines);
+		       my_hostname,
+				my_pid,
+				     spoolname,
+					 msgid[0] ? msgid : "<?>",
+					     articlename,
+						 msgsender[0] ? msgsender:"?",
+						     ok,
+							 article_lines);
 	if (fclose(fp) == EOF) {
 	    log_msg(L_ERRno,"can't write to logfile: %s",logfile);
 	}
@@ -580,35 +606,69 @@ log_to_file(char *articlename, char *ok)
 char *
 submit_article(char *articlename)
 {
-    FILE *art;
+    ARTHANDLE *article;
     long age;
     char *ok;
     char *id;
     char *to;
     char *reason;
-    char fullname[PATH_MAX];
-
-    /* work with full and partial paths */
-    build_alt_filename(fullname,spooldir,articlename,NULL);
+    int is_api = 0;
 
     progtitle("post: reading article");
-    log_msg(L_DEBUG,"reading %s", fullname);
 
-    if ((art = fopen(fullname, "r")) == NULL) {
+    if (IsToken(articlename)) {
+	/* Storage-API */
+	TOKEN art_token;
+
+	is_api = 1;
+	art_token = TextToToken(articlename);
+
+	log_msg(L_DEBUG,"reading API %s", articlename);
+
+	if (!SM_open) {
+	    BOOL value = 0; /* only need read/only access */
+
+	    SMsetup(SM_RDWR,(void *)&value);
+	    if (!SMinit()) {
+		log_msg(L_ERR,"can't initialize storage-API: %s",SMerrorstr);
+		return "Storage-API error";
+	    }
+	    SM_open = 1;
+	}
+	article = SMretrieve(art_token, RETR_ALL);
+
+    } else if (articlename[0] == '@' && 
+	       articlename[strlen(articlename)] == '@') {
+	/* storage-API type of entry, but no support */
+	log_msg(L_ERR,"no storage-API support for \"%s\"",articlename);
+	return "No support for storage-API";
+
+    } else {
+	/* work with full and partial paths */
+	char fullname[PATH_MAX];
+
+	build_alt_filename(fullname,spooldir,articlename,NULL);
+
+	log_msg(L_DEBUG,"reading %s", fullname);
+
+	article = SIMretrieve(fullname);
+    }
+
+    if (!article) {
 	/* article is missing - throw it away */
-	log_msg(L_ERRno,"can't find article \"%s\"",fullname);
+	log_msg(L_ERR,"can't find article \"%s\": %s",articlename,SMerrorstr);
 	++missing_articles;
 	ok = "Missing";
     } else {
 	if (ihave_opt) {
 	    /* use ihave */
-	    if (!(id = sniff_article(art))) {
-		log_msg(L_ERR,"article \"%s\" lacks a message ID",fullname);
+	    if (!(id = sniff_article(article))) {
+		log_msg(L_ERR,"article \"%s\" lacks a message ID",articlename);
 		++missing_articles;
 		ok = "Missing-ID";
 	    } else {
 		ihave_article(id);
-		ok = put_article(art,&reason);
+		ok = put_article(article,&reason);
 	    }
 	} else {
 	    /* request post */
@@ -616,7 +676,7 @@ submit_article(char *articlename)
 	    post_article();
 
 	    /* and do it */
-	    ok = put_article(art,&reason);
+	    ok = put_article(article,&reason);
 	}
     }
 
@@ -625,16 +685,20 @@ submit_article(char *articlename)
     } else {
 	if (failtime) {
 	    /* timeout message... */
-	    age = how_old(art,fullname);
+	    time_t now;
+
+	    time(&now);
+	    age = now - article->arrived;
+
 	    log_msg(L_DEBUG,"article is %ds old, limit is %ds", age, failtime);
 	    if (age > failtime) {
-		log_msg(L_ERR,"failed article \"%s\" killed",fullname);
+		log_msg(L_ERR,"failed article \"%s\" killed",articlename);
 		ok = "Failed";
 		if (!bounce || !bounce[0] || strcmp(bounce,"poster")==0) {
 		    /* return to sender */
 		    if (!msgsender[0]) {
 			log_msg(L_ERR,"no From-specification in \"%s\"",
-								    fullname);
+							      articlename);
 			to = 0;
 		    } else {
 			to = msgsender;
@@ -648,17 +712,31 @@ submit_article(char *articlename)
 		if (to) {
 		    ++bounced_articles;
 		    log_msg(L_ERR,"bounce message to %s",to);
-		    if (!bounce_msg(to,art,fullname,msgid,reason)) {
-			/* BUG: what shall we do? */
+		    if (!bounce_msg(to,article->data,article->len,
+				  article->arrived,articlename,msgid,reason)) {
+			/* BUG: what ever shall we do? */
 			log_msg(L_ERR,"bounce failed");
 		    }
 		}
 	    }
 	}
     }
-    if (art) fclose(art);
+    if (article) {
+	if (is_api) SMfreearticle(article);
+	else SIMfreearticle(article);
+    }
 
     return ok;
 }
 
-
+/*
+ *  close article interface
+ */
+void
+close_article(void)
+{
+    if (SM_open) {
+	SMshutdown();
+	SM_open = 0;
+    }
+}
