@@ -1,4 +1,4 @@
-/*  VER 159   TAB P   $Id: request.c,v 1.7 1998/09/09 07:32:13 src Exp $
+/*  VER 163   TAB P   $Id: request.c,v 1.10 1999/03/11 07:30:01 src Exp $
  *
  *  keep track of outstanding NNTP requests
  *
@@ -6,6 +6,15 @@
  *  the GNU General Public License applies
  *
  *  $Log: request.c,v $
+ *  Revision 1.10  1999/03/11 07:30:01  src
+ *  Implemented check for spool free space
+ *
+ *  Revision 1.9  1999/03/09 11:43:57  src
+ *  Cosmetic fixes
+ *
+ *  Revision 1.8  1998/11/21 19:14:24  src
+ *  Added --filter option
+ *
  *  Revision 1.7  1998/09/09 07:32:13  src
  *  Version 1.1
  *
@@ -22,14 +31,15 @@
  *  outstanding request descriptor
  */
 struct req {
-    int (*req_fn)();
+    int (*req_fn)(char *, long, char *);
     long req_where;
+    char *req_group;
 } ;
 
-struct req req_queue[REQ_MAX];
-int req_n;
-int req_head;
-int req_tail;
+static struct req req_queue[REQ_MAX];
+static int req_n = 0;
+static int req_head = 0;
+static int req_tail = 0;
 
 /* 
  *  get status line from NNTP
@@ -47,12 +57,12 @@ get_status(void)
     if (++req_tail >= REQ_MAX) req_tail = 0;
 
     /* get status */
-    if (!get_server_nntp(status, sizeof(status))) {
+    if (!get_server_nntp_nf(status, sizeof(status))) {
 	/* timeout */
 	return 0;
     }
 
-    return (*rp->req_fn)(status, rp->req_where);
+    return (*rp->req_fn)(status, rp->req_where, rp->req_group);
 }
 
 /* 
@@ -60,7 +70,7 @@ get_status(void)
  *  return false if trouble
  */
 int 
-put_request(char *request, int (*fn)(), long where)
+put_request(char *request, int (*fn)(), long where,char *group)
 {
     struct req *rp;
     int req_max = REQ_MAX;
@@ -76,6 +86,7 @@ put_request(char *request, int (*fn)(), long where)
     rp = &(req_queue[req_head]);
     rp->req_fn    = fn;
     rp->req_where = where;
+    rp->req_group = group;
 
     ++req_n;
     if (++req_head >= REQ_MAX) req_head = 0;
