@@ -1,4 +1,4 @@
-/*  VER 101  TAB P   $Id: server.c,v 1.11 1998/09/09 07:32:13 src Exp $
+/*  VER 109  TAB P   $Id: server.c,v 1.15 1999/03/31 03:38:29 src Exp $
  *
  *  NNTP server interface
  *
@@ -6,6 +6,18 @@
  *  the GNU General Public License applies
  *
  *  $Log: server.c,v $
+ *  Revision 1.15  1999/03/31 03:38:29  src
+ *  Moved errno.h to common.h
+ *
+ *  Revision 1.14  1999/03/15 08:51:36  src
+ *  Nothing.
+ *
+ *  Revision 1.13  1999/03/11 07:30:01  src
+ *  Implemented check for spool free space
+ *
+ *  Revision 1.12  1998/11/21 19:14:24  src
+ *  Added --filter option
+ *
  *  Revision 1.11  1998/09/09 07:32:13  src
  *  Version 1.1
  *
@@ -25,8 +37,6 @@
 #include "statistics.h"
 #include "nntp.h"
 
-#include <errno.h>
-
 /*
  *  currently open socket pair, for read and write
  */
@@ -73,7 +83,7 @@ open_server(char *host,char *port)
 						
 	log_msg(L_DEBUGMORE,"running \"%s\"", connect_exec);
 	fflush(nntp_sock.w_str);                       
-	if ((sts = script(connect_exec,        
+	if ((sts = script(connect_exec,
 		       /* in */  fileno(nntp_sock.r_str),
 		       /* out */ fileno(nntp_sock.w_str))) != 0) {
 	    /* connect failed */
@@ -123,9 +133,22 @@ close_server(int urgent)
 
 /*
  *  read a line from server
+ *  flush out request queue first
  *  return 0 on timeout and other severe problems
  */
 int get_server_nntp(char *line, int size)
+{
+    /* since this request is not windowed, we must flush any request queue */
+    if (!flush_request()) return -1;
+
+    return get_server_nntp_nf(line, size);
+}
+
+/*
+ *  read a line from server, with no regard to request queue
+ *  return 0 on timeout and other severe problems
+ */
+int get_server_nntp_nf(char *line, int size)
 {
     int len;
 
