@@ -1,4 +1,4 @@
-/*  VER 126  TAB P   $Id: socket.c,v 1.7 1998/09/09 07:32:14 src Exp $
+/*  VER 128  TAB P   $Id: socket.c,v 1.9 1999/03/31 03:38:29 src Exp $
  *
  *  open a socket connection and read/write to nntp server
  *
@@ -6,6 +6,12 @@
  *  the GNU General Public License applies
  *
  *  $Log: socket.c,v $
+ *  Revision 1.9  1999/03/31 03:38:29  src
+ *  Moved errno.h to common.h
+ *
+ *  Revision 1.8  1998/11/22 18:02:32  src
+ *  Filtering repaired
+ *
  *  Revision 1.7  1998/09/09 07:32:14  src
  *  Version 1.1
  *
@@ -21,7 +27,6 @@
 #include "options.h"
 #include "nntp.h"
 
-#include <errno.h>
 #include <signal.h>
 #include <setjmp.h>
 
@@ -235,7 +240,10 @@ get_socket(char *line, int size, SOCKET_D *sock)
 	if (read_telnet(line, size, sock) < 0) ok = 0;
     } else {
 	/* ver0.8PL1 by "J. Richard Sladkey" <jrs@foliage.com> */
-	if (fgets(line, size, sock->r_str) == NULL) ok = 0; 
+	if (fgets(line, size, sock->r_str) == NULL) {
+	    log_msg(L_ERRno,"read error on socket");
+	    ok = 0;
+	}
     }
 
     /* reset the alarm */
@@ -247,7 +255,7 @@ get_socket(char *line, int size, SOCKET_D *sock)
 
     /* report any error */
     if (ferror(sock->r_str)) {
-	log_msg(L_ERRno,"read error on server socket");
+	log_msg(L_ERRno,"error on socket");
 	return 0;
     }
     return ok;
