@@ -1,4 +1,4 @@
-/*  VER 031   TAB P   $Id: statistics.c,v 1.8 1998/09/13 16:52:09 src Exp $
+/*  VER 036   TAB P   $Id: statistics.c,v 1.13 1999/04/02 03:56:59 src Exp $
  *
  *  an NNTP news exchange client
  *
@@ -6,6 +6,21 @@
  *  the GNU General Public License applies
  *
  *  $Log: statistics.c,v $
+ *  Revision 1.13  1999/04/02 03:56:59  src
+ *  Patch for proper English by Uli Zappe
+ *
+ *  Revision 1.12  1999/03/16 17:35:38  src
+ *  Version 1.4pre
+ *
+ *  Revision 1.11  1999/03/16 08:03:31  src
+ *  Renamed lock() to do_lock()
+ *
+ *  Revision 1.10  1998/11/22 08:00:08  src
+ *  Option --no-queue
+ *
+ *  Revision 1.9  1998/11/21 19:14:24  src
+ *  Added --filter option
+ *
  *  Revision 1.8  1998/09/13 16:52:09  src
  *  Lock for statistics file
  *
@@ -50,19 +65,19 @@ statistics(time_t starttime,time_t endtime)
 			   bounced_articles, bounced_articles==1 ? "":"s");
     }
     if (unavailable_groups) {
-	log_msg(L_INFO,"%d group%s were not available from this server",
-			   unavailable_groups, unavailable_groups==1 ? "":"s");
+	log_msg(L_INFO,"%d group%s not available from this server",
+			   unavailable_groups, unavailable_groups==1 ? " was":"s were");
     }
     if (unseen_groups) {
 	log_msg(L_INFO,"fetched %d group%s never seen before",
 			   unseen_groups, unseen_groups==1 ? "":"s");
     }
-    if (already_articles || history_articles || mfilter_articles) {
-	if (mfilter || mfilter_articles) {
+    if (already_articles || history_articles || (filter_articles+fake_articles)) {
+	if (mfilter || filter || (fake_articles+filter_articles)) {
 	    log_msg(L_INFO,"omitted %ld article%s already in spool, %ld as crosspost%s, %ld filtered",
 			   history_articles, history_articles==1 ? "":"s",
 			   already_articles, already_articles==1 ? "":"s",
-			   mfilter_articles);  
+			   fake_articles+filter_articles);
 	} else {
 	    log_msg(L_INFO,"omitted %ld article%s already in spool, %ld as crosspost%s",
 			   history_articles, history_articles==1 ? "":"s",
@@ -92,7 +107,7 @@ statistics(time_t starttime,time_t endtime)
 	 */
 	progtitle("locking statistics file");
 	build_filename(lockname,statfile,LOCK_SUFFIX,NULL,NULL);
-	lock_id = lock("",lockname);
+	lock_id = do_lock("",lockname,0);
 
 	if (!(f = fopen(statfile,"a"))) {
 	    log_msg(L_ERRno,"can't append to \"%s\"",statfile);
@@ -125,9 +140,9 @@ statistics(time_t starttime,time_t endtime)
 						       /* new in version 0.7 */
 							     failed_articles,
 							     bounced_articles);
-	if (mfilter || mfilter_articles)
+	if (mfilter || filter || filter_articles)
 	    fprintf(f,"F %4ld\n",
-			 mfilter_articles);
+			 filter_articles);
 	fclose(f);
 	unlock_one(lock_id);
     }
