@@ -1,4 +1,4 @@
-/*  VER 122  TAB P   $Id: sys.c,v 1.12 1998/09/11 09:17:44 src Exp $
+/*  VER 144  TAB P   $Id: sys.c,v 1.23 1999/04/24 18:13:31 src Exp $
  *
  *  handle the local sys or newsfeeds file
  *
@@ -6,6 +6,40 @@
  *  the GNU General Public License applies
  *
  *  $Log: sys.c,v $
+ *  Revision 1.23  1999/04/24 18:13:31  src
+ *  Allowing '@' in newsfeeds syntax, making a special case of it
+ *  in the group list
+ *
+ *  Revision 1.22  1999/04/24 17:55:51  src
+ *  Improved message re. syntax errors
+ *
+ *  Revision 1.21  1999/04/18 09:29:47  src
+ *  --groups with INN syntax
+ *
+ *  Revision 1.20  1999/04/17 07:10:05  src
+ *  Repaired --groups behaviour
+ *
+ *  Revision 1.19  1999/04/07 07:25:16  src
+ *  Fixed backslash-something
+ *
+ *  Revision 1.18  1999/03/24 03:53:02  src
+ *  Implemented "newsx" as magic exclude pattern
+ *
+ *  Revision 1.17  1999/03/09 11:43:57  src
+ *  Cosmetic fixes
+ *
+ *  Revision 1.16  1999/03/07 14:58:19  src
+ *  Read newsconfig supported. Storage API supported.
+ *
+ *  Revision 1.15  1998/11/24 17:25:42  src
+ *  Version 1.3pre2: Repaired INN mode hamdling of ME entry
+ *
+ *  Revision 1.14  1998/11/22 18:02:32  src
+ *  Filtering repaired
+ *
+ *  Revision 1.13  1998/11/22 14:00:44  src
+ *  Cleaned up sys_group() interface
+ *
  *  Revision 1.12  1998/09/11 09:17:44  src
  *  Check path consistency (--no-path) and length (--max-path)
  *  GNU style option --help, --version, --dry-run, changed --noxx to --no-xx
@@ -31,6 +65,7 @@
 #include "proto.h"
 #include "options.h"
 #include "news.h"
+#include "newsconfig.h"
 
 /* 
  *  globals for sys
@@ -41,8 +76,9 @@ typedef struct sys_grp {
     char name[1]; /* extend as required... */
 } SYS_GRP;
 
-SYS_GRP *sysgrp = 0;
-SYS_GRP *sys_me = 0;
+static SYS_GRP *sysgrp = 0;         /* newsfeed entry */
+static SYS_GRP *sys_me = 0;         /* ME entry for Cnews */
+static SYS_GRP *sys_extra = 0;      /* from command line */
 
 /*
  *  add group in sys-file
@@ -50,18 +86,17 @@ SYS_GRP *sys_me = 0;
  *  so the list will appear in the opposite sequence of the original
  */
 static void
-sys_group(char *group,int me,int not)
+sys_group(char *group,SYS_GRP **sys,int not)
 {
     int n; 
     SYS_GRP *sp;
-    SYS_GRP **spp;
+
+    group = skipsp(group);
+    if (!*group) return;
 
     n = sizeof(SYS_GRP) + strlen(group);
 
-    if (!(sp = malloc(n))) {
-	log_msg(L_ERRno,"out of memory");
-	exit_cleanup(1);
-    }
+    sp = xmalloc(n);
 
     /* fill in structure */
     sp->next = 0; 
@@ -69,9 +104,40 @@ sys_group(char *group,int me,int not)
     strcpy(sp->name,group);
 
     /* insert at top of list */
-    spp = (me) ? &sys_me : &sysgrp;
-    sp->next = *spp;
-    *spp = sp;
+    sp->next = *sys;
+    *sys = sp;
+}
+
+/*
+ *  add extra newsgroup exclusion
+ */
+void
+add_extra(char *grp_list)
+{
+    char *p;
+    int not;
+
+    for (;;) {
+	if ((p = strchr(grp_list,','))) *p = '\0';
+
+	grp_list = skipsp(grp_list);
+	if (*grp_list == '\\') 
+	    ++grp_list; /* occurs due to bash behaviour */
+	if (*grp_list == '!' || *grp_list == '@') {
+	    ++grp_list;
+	    not = 1;
+	} else {
+	    not = 0;
+	}
+	log_msg(L_DEBUGMORE,"add --groups %s \"%s\"",not?"exclude":"include",
+						 grp_list);
+	sys_group(grp_list,&sys_extra,not);
+
+	if (!p) break; /* last one? */
+
+	*p = ',';
+	grp_list = p+1;
+    }
 }
 
 /*
@@ -95,9 +161,9 @@ sys_match(char *group,char *pattern)
 	}
     }
 
-    /* non-wildcard case */
+    /* not "all" */
     while (*group) {
-	if (*pattern== '\\' ) 
+	if (*pattern== '\\' )
 	    ++pattern;
 	if (*group == '.') {
 	    switch (*pattern) {
@@ -119,7 +185,7 @@ sys_match(char *group,char *pattern)
     case '\0': 
 	return 100; /* exact match */
     case '.':
-	/* "alt.humor" when pattern is "alt.humor.all" does not match */
+	/* "alt.humor" when pattern "alt.humor.all" does not match */
     default:
 	return 0; /* no match */
     }
@@ -130,82 +196,106 @@ sys_match(char *group,char *pattern)
  *
  *  Behaviour for INN, from newsfeeds(5):
  *
- *  For example,  to  receive  all
- *  ``comp''  groups,  but  only  comp.sources.unix within the
- *  sources newsgroups, the following set of patterns  can  be
- *  used:
- *    comp.*,!comp.sources.*,comp.sources.unix
- *  There  are  three  things to note about this example.  The
- *  first is that the trailing ``.*'' is required.  The second
- *  is  that,  again, the result of the last match is the most
- *  important.  The third is that ``comp.sources.*'' could  be
- *  written  as  ``comp.sources*'' but this would not have the
- *  same effect if there were a ``comp.sources-only'' group.
+ *      For example,  to  receive  all
+ *      ``comp''  groups,  but  only  comp.sources.unix within the
+ *      sources newsgroups, the following set of patterns  can  be
+ *      used:
+ *          comp.*,!comp.sources.*,comp.sources.unix
+ *      There  are  three  things to note about this example.  The
+ *      first is that the trailing ``.*'' is required.  The second
+ *      is  that,  again, the result of the last match is the most
+ *      important.  The third is that ``comp.sources.*'' could  be
+ *      written  as  ``comp.sources*'' but this would not have the
+ *      same effect if there were a ``comp.sources-only'' group.
  * 
- * Further quote from newsfeeds(5):
- * There  is  also  a  way  to subscribe to a newsgroup nega-
- * tively.  That is to say, do not send this  group  even  if
- * the article is cross-posted to a subscribed newsgroup.  If
- * the first character of a pattern is an  atsign  ``@'',  it
- * means that any article posted to a group matching the pat-
- * tern will not be sent  even  though  the  article  may  be
- * cross-posted  to  a  group  which is subscribed.  The same
- * rules of precedence apply in that the last  match  is  the
- * one which counts.
+ *  Further quote from newsfeeds(5):
  *
- * Since we only use these patterns for deciding which groups
- * to fetch we should treat '@' and '!' the same way: do not
- * fetch the group. But we need to support the syntax -- PMM 08/1998
+ *      There  is  also  a  way  to subscribe to a newsgroup nega-
+ *      tively.  That is to say, do not send this  group  even  if
+ *      the article is cross-posted to a subscribed newsgroup.  If
+ *      the first character of a pattern is an  atsign  ``@'',  it
+ *      means that any article posted to a group matching the pat-
+ *      tern will not be sent  even  though  the  article  may  be
+ *      cross-posted  to  a  group  which is subscribed.  The same
+ *      rules of precedence apply in that the last  match  is  the
+ *      one which counts.
+ *
+ *  Since we only use these patterns for deciding which groups
+ *  to fetch we should treat '@' and '!' the same way: do not
+ *  fetch the group. But we need to support the syntax -- PMM 08/1998
  */
 static int 
-sys_allow_grp(char *group, SYS_GRP *sp)
+sys_allow_grp_inn(char *group, SYS_GRP *sys_grp)
 {
-    if (inn_opt) {
-	/*
-	 * INN:
-	 * the last match is the most important
-	 * since our version of the list is stored with the last 
-	 * item first, we can get away by simply returning the 
-	 * status of *our* first match.
-	 */
-	for (; sp; sp = sp->next) {
-	  if (wildmat(group, sp->name)) {
+    SYS_GRP *sp = sys_grp;
+    int nots_only = 1;
+
+    /*
+     * INN:
+     * the last match is the most important
+     * since our version of the list is stored with the last
+     * item first, we can get away by simply returning the
+     * status of *our* first match.
+     */
+    for (; sp; sp = sp->next) {
+	if (!(sp->not)) nots_only = 0;
+	if (wildmat(group, sp->name)) {
 	    /* if !-flag or @-flag, then not allowed, otherwise all right */
 	    return !(sp->not);
-	  }
 	}
-	return 0; /* default to a no-no */
-    } else {
-	/*
-	 * C News:
-	 * the order is not significant
-	 * when more than one pattern matches:
-	 * if the longest matched pattern is longer than the longest
-	 * mismatched pattern, it is a match. otherwise, it is not.
-	 * longest is measured as number of words, "all" counting 
-	 * sligthly less than other words
-	 */
-	int n;
-	int best_n = 0;
-	int result = 0;
+    }
+    if (nots_only && sys_grp == sys_extra) {
+	/* command line with exclusions only */
+	/* default to a yes, otherwise it would have been meaningless */
+	return 1;
+    }
+    return 0; /* default to a no-no */
+}
 
-	for (; sp; sp = sp->next) {
-	    if ((n=sys_match(group,sp->name))) {
-		if (sp->not) {
-		    if (n >= best_n) {
-			best_n = n;
-			result = 0;
-		    }
-		} else {
-		    if (n > best_n) {
-			best_n = n;
-			result = 1;
-		    }
+/*
+ *  is group allowed in sys-file?
+ *
+ *  C News:
+ *  the order is not significant.
+ *  when more than one pattern matches:
+ *  if the longest matched pattern is longer than the longest
+ *  mismatched pattern, it is a match. otherwise, it is not.
+ *  longest is measured as number of words, "all" counting
+ *  sligthly less than other words
+ */
+static int 
+sys_allow_grp_cnews(char *group, SYS_GRP *sp)
+{
+    int n;
+    int best_n = 0;
+    int result = 0;
+
+    for (; sp; sp = sp->next) {
+	if ((n=sys_match(group,sp->name))) {
+	    if (sp->not) {
+		if (n >= best_n) {
+		    best_n = n;
+		    result = 0;
+		}
+	    } else {
+		if (n > best_n) {
+		    best_n = n;
+		    result = 1;
 		}
 	    }
 	}
-	return result;
     }
+    return result;
+}
+
+/*
+ *  is group allowed in sys-file?
+ */
+static int 
+sys_allow_grp(char *group, SYS_GRP *sys_grp)
+{
+    return (inn_opt) ? sys_allow_grp_inn(group, sys_grp)
+		     : sys_allow_grp_cnews(group, sys_grp);
 }
 
 /*
@@ -214,13 +304,16 @@ sys_allow_grp(char *group, SYS_GRP *sp)
 int 
 sys_allow(char *group)
 {
-    if (!sys_allow_grp(group, sys_me)) {
+    if (sys_extra && !sys_allow_grp_inn(group, sys_extra)) {
+	log_msg(L_DEBUGMORE,"group %s excluded by --groups",group);
+	return 0;
+    }
+    if (sys_me && !sys_allow_grp(group, sys_me)) {
 	log_msg(L_DEBUGMORE,"group %s not accepted by ME",group);
 	return 0;
     }
     if (!sys_allow_grp(group, sysgrp)) {
-	log_msg(L_DEBUGMORE,"group %s disallowed by %s file",group,
-					    inn_opt ? "newsfeeds":"sys");
+	log_msg(L_DEBUGMORE,"group %s disallowed by %s",group, cfg_newsfeeds);
 	return 0;
     }
     return 1;
@@ -237,15 +330,20 @@ sys_allow(char *group)
  *      "#" 
  *      "\n"
  *      "@" now added -- PMM
+ *
+ *  it turns out the '@'-sign has a special meaning only in group names
+ *  this syntax is so ugly it can make you sick, but we try to constrain
+ *  ourselves and try to make the best out of it
  */
 static void 
 sys_item(char *item)
 {
     static int state = 0;
-    static int is_me = 0;
+    static int lineno = 1;
+    static SYS_GRP **sys = &sysgrp;
 
-    switch (state) {
-	/* 
+    switch (state) { 
+	/*
 	 *  state: site
 	 */
     case 0:
@@ -257,28 +355,47 @@ sys_item(char *item)
 	case '\n':
 	    break;
 	case '!':
-	case '@':
 	case '/':
 	case ',':
 	case ':':
 	    /* error */
 	  bad_syntax:
-	    log_msg(L_ERR,"bad syntax in sys, '%s' not expected (state %d)",
-					       item,                state);
+	    log_msg(L_ERR,"bad syntax in %s line %d , '%s' not expected (state %d)",
+			      cfg_newsfeeds, lineno, item,                 state);
 	    state = 1;
 	    break;
 	default:
 	    if (strcmp(item,"ME") == 0) {
-		log_msg(L_DEBUGMORE,"found ME sys-entry");
+		/*
+		 * the handling of ME is differs significantly depending on
+		 * which news system we are using.
+		 *
+		 * For Cnews, from newssys(5):
+		 *   The ME line, or a line whose system
+		 *   name is explicitly that of the machine news is running on,
+		 *   has a rather different meaning from that of the other  sys
+		 *   file  lines:  its  subscriptions  subfield  identifies the
+		 *   newsgroups that this site subscribes to (i.e.  is  willing
+		 *   to  receive)
+		 *
+		 * BUG: explicit machine name from whoami not implemented.
+		 *
+		 * For INN, from newsfeeds(5):
+		 *   If the ME entry has a subscription list, then that list is
+		 *   automatically  prepended  to  the subscription list of all
+		 *   other entries.
+		 */
+		log_msg(L_DEBUGMORE,"found ME in %s", cfg_newsfeeds);
 		state = 2;
-		is_me = 1;
+		if (inn_opt) sys = &sysgrp; /* INN: prepend to general entry */
+		else sys = &sys_me;         /* Cnews: local subscription */
 	    } else if (strcmp(item,spoolname) == 0) {
-		log_msg(L_DEBUG,"found sys-entry for %s",item);
+		log_msg(L_DEBUG,"found entry for %s in %s", item, cfg_newsfeeds);
 		state = 2;
-		is_me = 0;
+		sys = &sysgrp;
 	    } else {
-		/* BUG: C News allows own system name instead of ME */
-		log_msg(L_DEBUGMORE,"skipping sys-entry for %s",item);
+		log_msg(L_DEBUGMORE,"skipping entry for %s in %s",
+							item, cfg_newsfeeds);
 		state = 1;
 	    }
 	    break;
@@ -306,7 +423,7 @@ sys_item(char *item)
 	case ':':
 	    /* only use this default if there are no exclusions */
 	    /* BUG: verify that this is correct */
-	    if (!is_me) add_exclusion(spoolname);
+	    if (sys == &sysgrp) add_exclusion(spoolname);
 	    state = 3;
 	    break;
 	case '#':
@@ -316,7 +433,7 @@ sys_item(char *item)
 	    state = 0;
 	    break;
 	case '/': /* a list of exclusions to follow */
-	    if (!is_me) state = 5;
+	    if (sys == &sysgrp) state = 5;
 	    break;
 	default:
 	    goto bad_syntax;
@@ -338,7 +455,6 @@ sys_item(char *item)
 	    state = 0;
 	    break;
 	case '!':
-	case '@':
 	case '/':
 	    goto bad_syntax;
 	case ',': /* ignore */
@@ -353,7 +469,8 @@ sys_item(char *item)
 	 *  state: groups
 	 */
     case 3:
-    case 4: /* seen a '!' */
+    case 4: /* seen a '!' or a '@' */
+    again:
 	switch (*item) {
 	case ':': /* ignore flags */
 	case '/': /* ignore distlist */
@@ -364,19 +481,24 @@ sys_item(char *item)
 	    state = 0;
 	    break;
 	case '!':
-	case '@':
 	    state = 4;
 	    break;
+	case '@':
+	    /* NOTE: '@' is not parsed as a special character */
+	    state = 4;
+	    ++item;
+	    goto again;
 	case ',':
 	    state = 3; 
 	    break;
 	default:
 	    log_msg(L_DEBUGMORE,"sys group %s%s",(state==4?"!":""),item);
-	    sys_group(item,is_me,state==4);
+	    sys_group(item,sys,state==4);
 	    state = 3;
 	    break;
 	}
     }
+    if (*item == '\n') ++lineno;
 }
 
 /*
@@ -424,7 +546,6 @@ sys_line(char *line, int offset)
 	case '#':
 	case '/':
 	case '!':
-	case '@':
 	case ':':
 	case ',':
 	case '\n':
@@ -471,9 +592,9 @@ sys_line(char *line, int offset)
 		if (start != line) strcpy(line,start);
 		return strlen(line); /* offset */
 	    default:
-		if (!start) start = p;
 		/* always include the next character as part of the token */
-		++p;
+		strcpy(p,p+1);
+		if (!start) start = p;
 		break;
 	    }
 	    break;
@@ -522,24 +643,17 @@ void
 load_sys(void)
 {
     FILE *f;
-    char sysname[PATH_MAX];
     char buf[BUFSIZ];
     int offset;
 
     progtitle("read sys");
 
-    if (inn_opt) {
-	build_alt_filename(sysname,news_home(),NEWSFEEDS,newsfeeds);
-    } else {
-	build_alt_filename(sysname,news_home(),C_SYS_FILE,newsfeeds);
-    }
-
-    if (!(f = fopen(sysname,"r"))) {
-	log_msg(L_ERRno,"can't open \"%s\"",sysname);
+    if (!(f = fopen(cfg_newsfeeds,"r"))) {
+	log_msg(L_ERRno,"can't open \"%s\"",cfg_newsfeeds);
 	exit_cleanup(8);
     }
-    log_msg(L_DEBUG,"reading %s",sysname);
-    if (!is_regular(f,sysname)) {
+    log_msg(L_DEBUG,"reading %s",cfg_newsfeeds);
+    if (!is_regular(f,cfg_newsfeeds,NULL)) {
 	fclose(f);
 	exit_cleanup(8);
     }
@@ -548,9 +662,14 @@ load_sys(void)
     while (fgets(buf+offset,BUFSIZ-offset,f)) {
 	offset = sys_line(buf,offset);
 	if (offset >= BUFSIZ-3) {
-	    log_msg(L_ERRno,"token too long in \"%s\"",sysname);
+	    log_msg(L_ERRno,"token too long in \"%s\"",cfg_newsfeeds);
 	    exit_cleanup(8);
 	}
     }
     fclose(f);
+
+    /* check it they specified the special newsx path */
+    newsx_path = path_match(NEWSX_PATH);
+    if (newsx_path)
+	log_msg(L_DEBUGMORE,"special %s exclude detected",NEWSX_PATH);
 }
