@@ -1,4 +1,4 @@
-/*  VER 149  TAB P   $Id: telnet.c,v 1.7 1998/09/09 07:32:14 src Exp $
+/*  VER 153  TAB P   $Id: telnet.c,v 1.9 1999/03/31 05:53:46 src Exp $
  *
  *  implement a very simple telnet protocol
  *
@@ -6,6 +6,12 @@
  *  the GNU General Public License applies
  *
  *  $Log: telnet.c,v $
+ *  Revision 1.9  1999/03/31 05:53:46  src
+ *  Seperated MAXHEADERSIZE from NNTP_STRLEN
+ *
+ *  Revision 1.8  1999/03/31 03:38:29  src
+ *  Moved errno.h to common.h
+ *
  *  Revision 1.7  1998/09/09 07:32:14  src
  *  Version 1.1
  *
@@ -19,8 +25,8 @@
 #include "common.h"
 #include "proto.h"
 #include "nntp.h"
+#include "news.h" /* MAXHEADERSIZE */
 
-#include <errno.h>
 #include <signal.h>
 #include <setjmp.h>
 
@@ -54,7 +60,7 @@ write_telnet(char *line, SOCKET_D *sock)
     int len;
     int n;
     char c1,c2;
-    char echo[NNTP_STRLEN+3];
+    char echo[MAXHEADERSIZE+3];
     int wrong_echo = 0;
 
     /* BUG: should escape IACs */
@@ -67,8 +73,8 @@ write_telnet(char *line, SOCKET_D *sock)
     if (sock->is_echo) {
       again:
 	/* await echo */
-	echo[NNTP_STRLEN+2] = '\0';
-	if (read_telnet(echo,NNTP_STRLEN+2,sock) < 0) return 0;
+	echo[MAXHEADERSIZE+2] = '\0';
+	if (read_telnet(echo,MAXHEADERSIZE+2,sock) < 0) return 0;
 	for (n=0; n<len; ++n) {
 	    c1 = line[n];  
 	    c2 = echo[n];  
@@ -150,11 +156,13 @@ read_telnet(char *line, int size, SOCKET_D *sock)
     int c;
     int d;
     int is_cr = 0;
-    static unsigned char buf[NNTP_STRLEN];
+    static unsigned char *buf = 0;
     static int buf_n = 0;
     static int buf_m = 0;
     static int not_first_time = 0;
 
+    if (!buf) buf = xmalloc(MAXHEADERSIZE);
+
     if (!not_first_time) {
 	not_first_time = 1;
 	/* IAC DO TELOPT_BINARY */
@@ -177,7 +185,7 @@ read_telnet(char *line, int size, SOCKET_D *sock)
 	while (buf_n >= buf_m) {
 	    /* need to refill local buffer */
 	    buf_n = 0;
-	    buf_m = recv(sock->r_fd,buf,NNTP_STRLEN,0);
+	    buf_m = recv(sock->r_fd,buf,MAXHEADERSIZE,0);
 	    if (buf_m < 0) {
 		log_msg(L_ERRno,"read error on telnet socket");
 		return -1;
