@@ -1,4 +1,4 @@
-/*  VER 038   TAB P   $Id: util.c,v 1.10 1998/09/14 06:30:25 src Exp $
+/*  VER 049   TAB P   $Id: util.c,v 1.14 1999/04/07 08:02:11 src Exp $
  *
  *  file utilities     
  *
@@ -6,6 +6,18 @@
  *  the GNU General Public License applies
  *
  *  $Log: util.c,v $
+ *  Revision 1.14  1999/04/07 08:02:11  src
+ *  Implemented --profile
+ *
+ *  Revision 1.13  1999/03/24 03:53:02  src
+ *  Implemented "newsx" as magic exclude pattern
+ *
+ *  Revision 1.12  1999/03/11 07:30:01  src
+ *  Implemented check for spool free space
+ *
+ *  Revision 1.11  1999/03/07 14:58:19  src
+ *  Read newsconfig supported. Storage API supported.
+ *
  *  Revision 1.10  1998/09/14 06:30:25  src
  *  *** empty log message ***
  *
@@ -29,15 +41,6 @@
 #include "options.h"
 #include "stat.h"
 
-/*
- *  the name of the news home directory
- */
-char *
-news_home(void)
-{                               
-    return (newshome && newshome[0]) ? newshome : NEWSHOME;
-}
-
 /*
  *  append the contents of a file to another file
  *  returns 0 on error, 1 if all OK
@@ -86,39 +89,12 @@ rename_file(char *from,char *to)
     return 1;
 }
 
-/*
- *  find how old a file is, in seconds
- */
-time_t
-what_date(FILE *f,char *name)
-{
-    struct stat st;
-
-    /* verify that it is a regular file */
-    if (fstat(fileno(f),&st) < 0) {
-	log_msg(L_ERRno,"can't fstat \"%s\"", name);
-	return 0;
-    }
-    return st.st_mtime;
-}
-
-/*
- *  find how old a file is, in seconds
- */
-time_t
-how_old(FILE *f,char *name)
-{
-    time_t now;
-    time(&now);
-
-    return now - what_date(f,name);
-}
-
 /*
  *  check if it is a regular file
+ *  as an added bonus, sets file size is desired
  */
 int 
-is_regular(FILE *f, char *name)
+is_regular(FILE *f, char *name, long *sizep)
 {
     struct stat st;
 
@@ -132,6 +108,7 @@ is_regular(FILE *f, char *name)
 	log_msg(L_ERR,"\"%s\" is not a regular file", name);
 	return 0;
     }
+    if (sizep) *sizep = (long)(st.st_size);
     return 1;
 }
 
@@ -199,6 +176,16 @@ text_time(time_t t)
     return buf;
 }
 
+/*
+ *  skip blanks
+ */
+char *
+skipsp(char *p)
+{
+    while (*p==' ' || *p=='\t') ++p;
+    return p;
+}
+
 /*
  *  memory allocation - never fail
  */
